import unittest

import pytest

from totalopenstation.formats.leica_tcr_705 import FormatParser

from . import BaseTestOutput


class TestLeicaTCR705Parser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/leica_tcr_705') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[1].geometry.x, 1002.825)
        self.assertAlmostEqual(self.fp.points[1].geometry.y, 999.529)
        self.assertAlmostEqual(self.fp.points[1].geometry.z, 98.430)

    def test_feature(self):
        self.assertEqual(self.fp.points[1].id, '101')
        self.assertEqual(self.fp.points[1].desc, 'WALL01')
        self.assertEqual(self.fp.points[0].desc, 'WALL01')


class TestLeicaTCR705Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/leica_tcr_705') as testdata:
            self.fp = FormatParser(testdata.read())



import unittest

import pytest

from totalopenstation.formats.zeiss_r5 import FormatParser

from . import BaseTestOutput


class TestZeissR5(unittest.TestCase):
    def setUp(self):
        with open('sample_data/zeiss_elta_r55/zeiss_elta_r55-R5.tops') as testdata:
            self.fp = FormatParser(testdata.read())
            self.feature = self.fp.points[11]
            self.point = self.feature.geometry

    def test_point(self):
        self.assertAlmostEqual(self.point.x, 6.554)
        self.assertAlmostEqual(self.point.y, 50.896)
        self.assertAlmostEqual(self.point.z, 11.334)

    def test_feature(self):
        self.assertEqual(self.feature.id, '1108')
        self.assertEqual(self.feature.desc, '67R')


class TestZeissR5Output(BaseTestOutput):
    @pytest.fixture
    def setup(self):
        with open('sample_data/zeiss_elta_r55/zeiss_elta_r55-R5.tops') as testdata:
            self.fp = FormatParser(testdata.read())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_csv.py
# Copyright 2019 Stefano Costa <steko@iosa.it>
# Copyright 2019 Damien Gaignon <damien.gaignon@gmail.com>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import csv
import io

from . import Builder


class OutputFormat(Builder):

    """
    Exports points data in CSV format.

    ``data`` should be an iterable containing Feature objects.
    """

    def __init__(self, data):
        self.data = data
        self.output = io.StringIO()
        fieldnames = ['pid', 'type', 'point_name', 'x', 'y', 'z', 'angle', 'z_angle', 'distance',
                      'th', 'ih', 'circle', 'station']
        self.writer = csv.DictWriter(self.output, quoting=csv.QUOTE_NONNUMERIC, fieldnames=fieldnames)
        self.writer.writeheader()

    def process(self):

        for feature in self.data:
            row = {
                'pid': feature.id,
                'type': feature.desc,
                'x' : feature.geometry.x,
                'y': feature.geometry.y
            }

            try:  # not all input formats include z coordinates
                row['z'] = feature.geometry.z
            except ValueError:
                row['z'] = ''

            # a few cases with simple yes/no logic
            for prop in ['point_name', 'ih', 'circle', 'z_angle', 'th']:
                row[prop] = feature.properties.get(prop, '')  # empty string as default value

            # not all input formats include azimuth/angle
            row['angle'] = feature.properties.get('azimuth',
                                                  feature.properties.get('angle', ''))

            # not all input formats include distance
            row['distance'] = feature.properties.get('slope_dist',
                                                 feature.properties.get('horizontal_dist', ''))

            # not all input formats include station name
            row['station'] = feature.properties.get('st_name', '')

            self.writer.writerow(row)

        return self.output.getvalue()



import unittest

import pytest

from totalopenstation.formats.topcon_gts import FormatParser

from . import BaseTestOutput

class TestTopconGTSParser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/topcon_gts_229') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[0].geometry.x, -5247.0753003)
        self.assertAlmostEqual(self.fp.points[0].geometry.y, 5049.9025715)
        self.assertAlmostEqual(self.fp.points[0].geometry.z, 64289.3662938)

    def test_feature(self):
        self.assertEqual(self.fp.points[0].id, '2')
        self.assertEqual(self.fp.points[0].desc, '00099')
        self.assertEqual(self.fp.points[1].desc, '00101')

class TestTopconGTSOutput(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/topcon_gts_229') as testdata:
            self.fp = FormatParser(testdata.read())



# -*- coding: utf-8 -*-
# filename: formats/trimble_are.py
# Copyright 2009 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2009 Stefano Costa <steko@iosa.it>
# Copyright 2009 Alessandro Bezzi <alessandro.bezzi@arc-team.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Feature, Parser, Point


class FormatParser(Parser):

    def is_point(self, line):
        is_point = False
        if "5=" and "4=" and "37=" and "38=" and "39=" in line:
            is_point = True
        return is_point

    def get_point(self, chunk):
        tokens = {}
        rows = chunk.splitlines()
        for i in rows:
            if i.startswith('5='):
                tokens['n'] = i.split('=')[1]
            if i.startswith('4='):
                tokens['p'] = i.split('=')[1]
            if i.startswith('37='):
                tokens['x'] = i.split('=')[1]
            if i.startswith('38='):
                tokens['y'] = i.split('=')[1]
            if i.startswith('39='):
                tokens['z'] = i.split('=')[1]
        tokens['text'] = rows[0]

        try:
            p = Point(tokens['y'],
                      tokens['x'],
                      tokens['z'])
            f = Feature(p,
                        desc=tokens['p'],
                        id=tokens['n'])
        except KeyError:
            pass
        else:
            return f

    def split_points(self):
        splitted_points = self.data.split('0=')
        return splitted_points



import unittest

import pytest

from totalopenstation.formats.trimble_are import FormatParser

from . import BaseTestOutput


class TestTrimbleAREParser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/trimble/BSG-08-11-19.are') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[1].geometry.x, 499.622)
        self.assertAlmostEqual(self.fp.points[1].geometry.y, 497.857)
        self.assertAlmostEqual(self.fp.points[1].geometry.z, 1.348)

    def test_feature(self):
        self.assertEqual(self.fp.points[1].id, '3')
        self.assertEqual(self.fp.points[1].desc, 'FIX')
        self.assertEqual(self.fp.points[0].desc, 'TEST')


class TestTrimbleAREOutput(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/trimble/BSG-08-11-19.are') as testdata:
            self.fp = FormatParser(testdata.read())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_dxf.py
# Copyright 2008-2009 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


from . import Builder


class OutputFormat(Builder):

    """
    Exports points data in AutoCAD DXF format.

    It is based on the official DXF2000 documentation. Works with AutoCAD
    versions ranging at least from 2005 up to 2009, and QCAD.

    ``data`` should be an iterable (e.g. list) containing one iterable (e.g.
    tuple) for each point. The default order is PID, x, y, z, TEXT.

    This is consistent with our current standard.
    """

    def __init__(self, data, separate_layers=True):

        self.data = data
        self.separate_layers = separate_layers
        self.text_height = 0.05

    def process(self):
        '''Process the input data and return a string as output.

        This is because we want to keep the generation of output
        separated from saving it to disk.'''

        result = ''

        # header
        result += '999\nDXF created from Total Open Station\n'
        result += '  0\nSECTION\n'
        result += '  2\nHEADER\n'
        result += '  9\n$ACADVER\n'
        result += '  1\nAC1009\n' # R11
        result += '  0\nENDSEC\n'

        # extract layer list
        codes = set([p.desc for p in self.data])
        codes = [c.replace('.','_') for c in codes]
        layers = dict(enumerate(codes))
        colors = dict((i, j % 255) for i, j in zip(list(layers.values()), list(layers.keys())))

        # layer table
        result += '  0\nSECTION\n  2\nTABLES\n  0\nTABLE\n  2\nLAYER\n'
        for l in codes:
            if self.separate_layers is True:
                result += '  0\nLAYER\n'           # start definition of LAYER
                result += '  5\n10\n'              # LAYER handle
                result += '  2\n%s_POINTS\n' % l    # LAYER name
                result += ' 70\n0\n'              # LAYER is not frozen
                result += ' 62\n%s\n' % (int(colors[l]) + 1) # LAYER color
                result += '  6\nCONTINUOUS\n'      # LAYER linetype

                result += '  0\nLAYER\n'           # same as above
                result += '  5\n10\n'
                result += '  2\n%s_Z_COORDS\n' % l
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'

                result += '  0\nLAYER\n'           # ditto
                result += '  5\n10\n'
                result += '  2\n%s_LABELS\n' % l
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'
            else:
                result += '  0\nLAYER\n'           # ditto
                result += '  5\n10\n'
                result += '  2\n%s\n' % l          # LAYER name w/o any suffix
                result += ' 70\n0\n'
                result += ' 62\n%s\n' % (int(colors[l]) + 1)
                result += '  6\nCONTINUOUS\n'

        result += '  0\nENDTAB\n  0\nENDSEC\n'

        # drawing entities
        result += '  0\nSECTION\n  2\nENTITIES\n'

        for p in self.data:
            p_layer = p.desc
            geom = p.geometry
            if geom.geom_type == 'Point':
                if self.separate_layers is True:
                    layer_point = "%s_POINTS" % p_layer
                    layer_z_text = "%s_Z_COORD" % p_layer
                    layer_id_text = "%s_LABELS" % p_layer
                else:
                    layer_point = layer_z_text = layer_id_text = p_layer
                p_yz = str(float(geom.y) - (self.text_height * 1.2))

                # add point
                result += '  0\nPOINT\n'
                result += '  8\n%s\n' % layer_point
                result += ' 10\n%s\n' % geom.x
                result += ' 20\n%s\n' % geom.y

                # add ID number
                result += '  0\nTEXT\n'
                result += '  1\n%s\n' % p.id
                result += '  8\n%s\n' % layer_id_text
                result += ' 10\n%s\n' % geom.x
                result += ' 20\n%s\n' % geom.y
                result += ' 40\n%01.2f\n' % self.text_height
                result += ' 62\n256\n'

                try:
                    geom.z
                except ValueError:
                    pass
                else:
                    # add Z value as string
                    result += '  0\nTEXT\n'
                    result += '  1\n%s\n' % geom.z
                    result += '  8\n%s\n' % layer_z_text
                    result += ' 10\n%s\n' % geom.x
                    result += ' 20\n%s\n' % p_yz
                    result += ' 40\n%01.2f\n' % self.text_height
                    result += ' 62\n256\n'

            elif geom.geom_type == 'LineString':
                result += '  0\nPOLYLINE\n'
                result += '  8\n%s\n' % p_layer
                result += '  6\nCONTINUOUS\n'
                result += ' 62\n256\n'
                result += ' 66\n1\n'
                result += ' 70\n0\n'
                for v in geom.coords:
                    result += '  0\nVERTEX\n'
                    result += '  8\n%s\n' % p_layer
                    result += ' 10\n%s\n' % v[0] # x
                    result += ' 20\n%s\n' % v[1] # y
                    try:
                        result += ' 30\n%s\n' % v[2] # z
                    except IndexError:
                        result += ' 30\n0\n'
                result += '  0\nSEQEND\n'
            else:
                raise NotImplementedError
        result += '  0\nENDSEC\n  0\nEOF\n'
        return result



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_txt.py
# Copyright 2008 Luca Bianconi<lc.bianconi@googlemail.com>
# Copyright 2008,2011 Stefano Costa <steko@iosa.it>
# Copyright 2015-2016 Damien Gaignon <damien.gaignon@gmail.com>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from math import sin,cos, pi


def deg_to_gon(angle):
    '''Convert degrees format to grade (gon) format.'''
    return angle * 400 / 360


def deg_to_rad(angle):
    '''Convert degrees format to radian format.'''
    return angle * (2 * pi) / 360


def deg_to_mil(angle):
    return angle * 6400 / 360


def deg_to_dms(angle):
    d, m = divmod(angle, 1)
    m, s = divmod(m * 100, 1)
    return d + m * 60 / 10000 + s * 100 * 3600 / 100000000


def dms_to_deg(angle):
    '''Convert degrees in DDD.MMSS format to degree format.'''
    d, m = divmod(angle, 1)
    m, s = divmod(m * 100, 1)
    return d + m / 60 + s * 100 / 3600


def dms_to_gon(angle):
    '''Convert degrees in DDD.MMSS format to grade (gon) format.'''
    angle = dms_to_deg(angle)
    return deg_to_gon(angle)


def dms_to_rad(angle):
    '''Convert degrees in DDD.MMSS format to radians format.'''
    angle = dms_to_deg(angle)
    return deg_to_rad(angle)


def dms_to_mil(angle):
    '''Convert degrees in DDD.MMSS format to radians format.'''
    angle = dms_to_deg(angle)
    return deg_to_mil(angle)


def rad_to_deg(angle):
    return angle * 360 / (2 * pi)


def rad_to_dms(angle):
    angle = rad_to_deg(angle)
    return deg_to_dms(angle)


def rad_to_gon(angle):
    return angle * 400 / (2 * pi)


def rad_to_mil(angle):
    return angle * 6400 / (2* pi)


def mil_to_gon(angle):
    '''Convert degrees in mil (NATO) format to grade (gon) format.'''
    return angle * 400 / 6400


def mil_to_rad(angle):
    '''Convert degrees in mil (NATO) format to radian format.'''
    return angle * (2 * pi) / 6400


def mil_to_deg(angle):
    return angle * 360 / 6400


def mil_to_dms(angle):
    angle = mil_to_deg(angle)
    return deg_to_dms(angle)


def gon_to_rad(angle):
    '''Convert grade (gon) format to radian format.'''
    return angle * (2 * pi) / 400


def gon_to_deg(angle):
    return angle * 360 / 400


def gon_to_dms(angle):
    angle = gon_to_deg(angle)
    return deg_to_dms(angle)


def gon_to_mil(angle):
    return angle * 6400 / 400


def to_rad(angle, angle_unit):
    '''Conversion function for angles to radian'''
    if angle_unit == "dms":
        return dms_to_rad(angle)
    elif angle_unit == "mil":
        return mil_to_rad(angle)
    elif angle_unit == "deg":
        return deg_to_rad(angle)
    else:
        return gon_to_rad(angle)


def to_deg(angle, angle_unit):
    '''Conversion function for angles to degree'''
    if angle_unit == "dms":
        return dms_to_deg(angle)
    elif angle_unit == "mil":
        return mil_to_deg(angle)
    elif angle_unit == "rad":
        return rad_to_deg(angle)
    else:
        return gon_to_deg(angle)


def to_gon(angle, angle_unit):
    '''Conversion function for angles to degree'''
    if angle_unit == "dms":
        return dms_to_gon(angle)
    elif angle_unit == "mil":
        return mil_to_gon(angle)
    elif angle_unit == "rad":
        return rad_to_gon(angle)
    else:
        return deg_to_gon(angle)


def to_dms(angle, angle_unit):
    '''Conversion function for angles to degree'''
    if angle_unit == "gon":
        return gon_to_dms(angle)
    elif angle_unit == "mil":
        return mil_to_dms(angle)
    elif angle_unit == "rad":
        return rad_to_dms(angle)
    else:
        return deg_to_dms(angle)


def to_mil(angle, angle_unit):
    '''Conversion function for angles to degree'''
    if angle_unit == "dms":
        return dms_to_mil(angle)
    elif angle_unit == "gon":
        return gon_to_mil(angle)
    elif angle_unit == "rad":
        return rad_to_mil(angle)
    else:
        return deg_to_mil(angle)


def vertical_to_zenithal(angle, angle_unit):
    '''Convert angle from vertical (reference is horizontal) to
    zenithal (reference is north)'''
    if angle_unit == "dms":
        return to_dms(90 - to_deg(angle, angle_unit), "deg")
    elif angle_unit == "gon":
        return 100 - angle
    elif angle_unit == "rad":
        return (pi / 2) - angle
    else:
        return 90 - angle

def horizontal_to_slope(dist, angle, angle_unit, angle_type="z"):
    '''Convert distance to slope from horizontal
    Angle is considered zenithal by default'''
    angle = to_rad(angle, angle_unit)
    if angle_type == "z":
        return dist / sin (angle)
    else:
        return dist / cos (angle)



#! /usr/bin/env python

__all__ = ["tops_csv", "tops_dxf", "tops_dat", "tops_sql", "tops_txt", "tops_geojson"]

class Builder:

    def __init__(self, data):
        """Init method which **must** be overridden in the child class
        to have a working builder.
        
        Args:
        data (:class:`formats.Parser`): A list of :class:`formats.Feature`
        """

        self.data = data
    
    def process(self):
        """Action for building the output string.
        
        This method **must** be overridden in the child class
        to have a working builder.
        
        Process the input data (processing data).
        This is because we want to keep the generation of output separated from
        saving it to disk.

        Return:
            str: A string representing the value to output.
        """

        pass


BUILTIN_OUTPUT_FORMATS = {
    'dxf': ('tops_dxf', 'OutputFormat', 'DXF'),
    'csv': ('tops_csv', 'OutputFormat', 'CSV'),
    'sql': ('tops_sql', 'OutputFormat', 'OGC-SQL'),
    'dat': ('tops_dat', 'OutputFormat', 'DAT'),
    'txt': ('tops_txt', 'OutputFormat', 'Text'),
    'geojson': ('tops_geojson', 'OutputFormat', 'GeoJSON'),
    }



import unittest

from totalopenstation.formats import Feature, Point
from totalopenstation.output.tops_csv import OutputFormat

class TestCSVOutput(unittest.TestCase):

    def setUp(self):
        self.data = [
            Feature(Point(12.8, 76.3, 56.2),
                    desc='PT',
                    point_name='TEST POINT',
                    id=1),
            Feature(Point(19.8, 26.3, 46.2),
                    desc='PT',
                    point_name='TEST POINT #2',
                    id=2),
        ]

    def test_output(self):
        self.output = OutputFormat(self.data).process()
        self.assertEqual(self.output.splitlines()[1], '1,"PT","TEST POINT",12.8,76.3,56.2,"","","","","","",""')



import unittest

import pytest

from totalopenstation.formats.nikon_raw_v200 import FormatParser

from . import BaseTestOutput


class TestNikonParser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/nikon_raw_v200/nikon_raw_v200.tops') as testdata:
            self.fp = FormatParser(testdata.read())
        with open('sample_data/nikon_raw_v200/nikon_dtm.tops') as testdata2:
            self.fp2 = FormatParser(testdata2.read())

    def test_points(self):
        self.assertAlmostEqual(self.fp.points[1].geometry.x, 0.0)

    def test_basepoint(self):
        self.assertAlmostEqual(self.fp.points[0].geometry.x, 0.0)
        self.assertAlmostEqual(self.fp.points[0].geometry.y, 0.0)
        self.assertAlmostEqual(self.fp2.points[7].geometry.x, 10000.08905808)
        self.assertAlmostEqual(self.fp2.points[7].geometry.y, 10000.30257267)

    def test_feature(self):
        self.assertEqual(self.fp.points[3].id, 3)
        self.assertEqual(self.fp.points[3].desc, 'PT')


class TestNikonRAWOutput(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/nikon_raw_v200/nikon_raw_v200.tops') as testdata:
            self.fp = FormatParser(testdata.read())


class TestNikonRAW2Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/nikon_raw_v200/nikon_dtm.tops') as testdata:
            self.fp = FormatParser(testdata.read())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_dat.py
# Copyright 2008-2010 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Builder


def to_dat(point):
    result = "{p.id} {p.id} {p.geometry.x} {p.geometry.y}\r\n".format(p=point)
    return result


class OutputFormat(Builder):

    """
    Exports points data in DAT format suitable for use with Archis.

    ``data`` must be an iterable containing Feature objects.
    """

    def __init__(self, data):
        self.data = data

    def process(self):
        lines = [to_dat(point) for point in self.data]
        output = "".join(lines)
        return output



# -*- coding: utf-8 -*-
# filename: formats/topcon_gts.py
# Copyright 2010 Stefano Costa <steko@iosa.it>
# Copyright 2010 Cristiano Moscaritolo <cristianomoscaritolo@yahoo.it>
# Copyright 2010 Olga Pastore <olga.pastore@gmail.com>
# Copyright 2010 Enza Battiante <enza.battiante@alice.it>
# Copyright 2010 Raffaele Fanelli <rfl.fanelli@libero.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from .polar import BasePoint, PolarPoint
from totalopenstation.formats.conversion import deg_to_gon
from . import Feature

logger = logging.getLogger(__name__)


class FormatParser:
    '''A FormatParser for Topcon GTS polar format.

    It doesn't inherit from the base Parser class because the internal
    procedure is quite different, but it implements the same API so it
    can work nicely with other parts of the library.'''

    def __init__(self, data):
        # workaround for (apparently) corrupt downloaded data
        clean_data = ''.join([l[1:-5] for l in data.splitlines()])
        self.rows = clean_data.split(',')

    @property
    def points(self):
        points = []
        bp = BasePoint(x=0, y=0, z=0, ih=0, b_zero_st=0.0 )
        for row in self.rows:
            fs = row.split('+')
            try:
                pid = fs[1][:-3]
            except IndexError:
                continue
            text = fs[-1][0:5]
            try:
                th = float(fs[0][:-1])
            except ValueError:
                continue
            coordorder = 'NEZ'
            dist = float(fs[2].split('m')[0])
            angle = deg_to_gon(float(fs[3][:-1]) / 10000)
            z_angle = deg_to_gon(float(fs[4][:-3]) / 10000)
            p = PolarPoint(angle_unit='gon',
                           z_angle_type='z',
                           dist_type='s',
                           dist=dist,
                           angle=angle,
                           z_angle=z_angle,
                           th=th,
                           base_point=bp,
                           pid=pid,
                           text=text,
                           coordorder=coordorder)
            f = Feature(p.to_point(),
                        desc=text,
                        id=pid)
            points.append(f)
        logger.debug(points)
        return points



import unittest

import pytest

from totalopenstation.formats.leica_gsi import FormatParser

from . import BaseTestOutput


class TestLeicaGSI16Parser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/leica_gsi/leica_gsi16_gurob.gsi') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[0].geometry.x, 8.0757244)
        self.assertAlmostEqual(self.fp.points[0].geometry.y, 11.21674196)
        self.assertAlmostEqual(self.fp.points[0].geometry.z, -0.2890493)

    def test_feature(self):
        self.assertEqual(self.fp.points[0].id, 2)
        self.assertEqual(self.fp.points[0].point_name, 'GDEM5415')
        self.assertEqual(self.fp.points[0].desc, 'PT')
        self.assertEqual(self.fp.points[1].point_name, 'GDEM5416')
        self.assertEqual(self.fp.points[0].desc, 'PT')

    def test_linestring(self):
        self.ls = self.fp.build_linestring()
        self.assertAlmostEqual(self.ls.coords[0][0], 8.0757244)


class TestLeicaGSI16Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/leica_gsi/leica_gsi16_gurob.gsi') as testdata:

            self.fp = FormatParser(testdata.read())


class TestLeicaGSI8Parser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/leica_gsi/leica_gsi8_ertola.gsi') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[0].geometry.x, 515.836)
        self.assertAlmostEqual(self.fp.points[0].geometry.y, 525.871)
        self.assertAlmostEqual(self.fp.points[0].geometry.z, 3.079)

    def test_feature(self):
        self.assertEqual(self.fp.points[0].id, 1)
        self.assertEqual(self.fp.points[0].desc, 'PT')
        self.assertEqual(self.fp.points[0].point_name, '1')
        self.assertEqual(self.fp.points[1].desc, 'PT')
        self.assertEqual(self.fp.points[1].point_name, '2')

    def test_linestring(self):
        self.ls = self.fp.build_linestring()
        self.assertAlmostEqual(self.ls.coords[0][0], 515.836)
        self.assertAlmostEqual(self.ls.coords[3][2], 2.553)
        self.assertAlmostEqual(self.ls.bounds[0], -7.4660913)


class TestLeicaGSI8Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/leica_gsi/leica_gsi16_gurob.gsi') as testdata:
            self.fp = FormatParser(testdata.read())



# -*- coding: utf-8 -*-
# filename: formats/__init__.py
# Copyright 2008-2010 Stefano Costa <steko@iosa.it>
# Copyright 2008 Luca Bianconi <luxetluc@yahoo.it>
# Copyright 2015-2016 Damien Gaignon <damien.gaignon@gmail.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from pygeoif import geometry as g
from math import pi


logger = logging.getLogger(__name__).addHandler(logging.NullHandler())

class Point(g.Point):
    pass


class LineString(g.LineString):
    pass


class Feature(g.Feature):
    '''A GeoJSON-like Feature object.'''

    def __init__(self, geom, desc, id=None, **properties):
        g.Feature.__init__(self, geom, properties, feature_id=id)
        self.properties['desc'] = desc

    @g.Feature.geometry.setter
    def geometry(self, value):
        '''Set the geometry attribute.

        By default, geometry property return the geometry value.
        '''

        self._geometry = value

    @property
    def desc(self):
        '''Return the desc property
        '''

        return self.properties['desc']

    @property
    def point_name(self):
        '''Return the point_name property
        '''

        return self.properties['point_name']


class FeatureCollection(g.FeatureCollection):
    pass


class Parser:
    '''Parses a *single* string of raw data.

    This means that if you plan to load data from a file you have to
    pass the output of open(file).read() to this class.

    Args:
        data (str): A string representing the file to be parsed.
            
    Attributes:
        data (str): A string representing the file to be parsed **could**
            be overridden by the init method.
    '''

    def __init__(self, data):
        """Init method which **should** be overridden in the child class
        to have a working parser."""

        self.data = data

    def is_point(self, line):
        """Action for finding which parts of the source file are points.

        This method **must** be overridden in the child class
        to have a working parser.
        
        Returns:
            A boolean
        """

        pass

    def get_point(self, line):
        """Action for getting points from source file.

        This method **must** be overridden in the child class
        to have a working parser.
        
        Returns:
            A :class:`formats.Feature` object."""

        pass

    def split_points(self):
        """Action for splitting points.

        Defaults to ``splitlines()`` because most formats have one
        point per line.

        Override this method if the format is different."""

        return self.data.splitlines()

    def build_linestring(self):
        '''Join all Point objects into a LineString.
        
        Returns:
            A :class:`formats.LineString` object.
        '''

        return LineString([f.geometry for f in self.points])

    @property
    def points(self):
        """Action for parsing a source file and for finding points.

        This method **could** be overridden in the child class
        to have a working parser.
        
        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.
        """


        self.d = self.split_points()

        valid_lines = filter(self.is_point, self.d)
        fg_lines = map(self.get_point, valid_lines)

        return [p for p in fg_lines if p is not None]

    @property
    def raw_line(self):
        """Action for parsing a source file and for retrieving raw data.

        This method **must** be overridden in the child class
        to have a working parser.
        
        Returns:
            A list of GeoJSON-like Feature object representing
            representing raw data i.e. polar coordinates and other
            informations.
        """

        pass


def check_coordorder(coordorder):
    '''Check if coordinates order is valid.

    Args:
        coordorder (str): A string representing the type of coordinates i.e.
                NEZ or ENZ.
    '''

    if any((coordorder == v for v in COORDINATE_ORDER)):
        return coordorder
    else:
        logger.info('Invalid coordinate order')

BUILTIN_INPUT_FORMATS = {
    'carlson_rw5': ('carlson_rw5', 'FormatParser', 'Carlson RW5'),
    'leica_gsi': ('leica_gsi', 'FormatParser', 'Leica GSI'),
    'leica_tcr_705': ('leica_tcr_705', 'FormatParser', 'Leica TCR 705'),
    'leica_tcr_1205': ('leica_tcr_1205', 'FormatParser', 'Leica TCR 1205'),
    'nikon_raw_v200': ('nikon_raw_v200', 'FormatParser','Nikon RAW V2.00'),
    'sokkia_sdr33': ('sokkia_sdr33', 'FormatParser', 'Sokkia SDR33'),
    'topcon_gts': ('topcon_gts', 'FormatParser', 'Topcon GTS'),
    'trimble_are': ('trimble_are', 'FormatParser', 'Trimble AREA'),
    'zeiss_r5': ('zeiss_r5', 'FormatParser', 'Zeiss R5'),
    'zeiss_rec_500': ('zeiss_rec_500', 'FormatParser', 'Zeiss REC 500'),
    }

UNITS_CIRCLE = {
    'dms': 360,
    'deg': 360,
    'gon': 400,
    'mil': 6400,
    'rad': 2 * pi,
    }

UNKNOWN_STATION = Point(10000, 10000, 100)
UNKNOWN_POINT = Point(-1, -1, -1)

COORDINATE_ORDER = ('NEZ', 'ENZ')



import unittest

import pytest

from totalopenstation.formats.leica_tcr_1205 import FormatParser

from . import BaseTestOutput


class TestLeicaTCR1205Parser(unittest.TestCase):

    def setUp(self):
        with open('sample_data/leica_tcr_1205') as testdata:
            self.fp = FormatParser(testdata.read())

    def test_point(self):
        self.assertAlmostEqual(self.fp.points[1].geometry.x, 450402.042)
        self.assertAlmostEqual(self.fp.points[1].geometry.y, 205885.618)
        self.assertAlmostEqual(self.fp.points[1].geometry.z, 61.309)

    def test_feature(self):
        self.assertEqual(self.fp.points[1].id, '10001')
        self.assertEqual(self.fp.points[1].desc, 'Line0001')
        self.assertEqual(self.fp.points[2].desc, 'Line0001')

    def test_linestring(self):
        self.ls = self.fp.build_linestring()
        self.assertAlmostEqual(self.ls.coords[6][0], 450403.738)
        self.assertAlmostEqual(self.ls.coords[6][1], 205883.360)
        self.assertAlmostEqual(self.ls.coords[6][2], 61.318)


class TestLeicaTCR1205Output(BaseTestOutput):

    @pytest.fixture
    def setup(self):
        with open('sample_data/leica_tcr_1205') as testdata:
            self.fp = FormatParser(testdata.read())



# -*- coding: utf-8 -*-
# filename: polar.py
# Copyright 2010, 2014 Stefano Costa <steko@iosa.it>
# Copyright 2015-2016 Damien Gaignon <damien.gaignon@gmail.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from math import cos, sin, tan

from totalopenstation.formats.conversion import to_rad
from . import Feature, Point, UNITS_CIRCLE


def polar_to_cartesian(angle_unit, z_angle_type, dist_type, base_x, base_y, base_z, dist, azimuth, z_angle, ih, th):
    '''Convert polar coordinates to cartesian.

    Needs base point coordinates, measurement angles and distance.
    Angles must be given in radians.

    Some important caveats of the current implementation:

    - the horizontal ``angle`` is hardcoded with zero azimuth at North
    '''

    if z_angle_type == 'dh':
        dZ = z_angle
    if z_angle_type == 'v':
        if dist_type == 'h':
            dZ = dist * tan(to_rad(z_angle, angle_unit))
        else:
            dZ = dist * sin(to_rad(z_angle, angle_unit))
    if z_angle_type == 'z':
        if dist_type == 'h':
            dZ = dist / tan(to_rad(z_angle, angle_unit))
        else:
            dZ = dist * cos(to_rad(z_angle, angle_unit))
    if dist_type == 's':
        if z_angle_type == 'dh':
            dist = (dist ** 2 - z_angle ** 2) ** (0.5)
        if z_angle_type == 'v':
            dist = dist * cos(to_rad(z_angle, angle_unit))
        if z_angle_type == 'z':
            dist = dist *  sin(to_rad(z_angle, angle_unit))
    dX = dist * sin(to_rad(azimuth, angle_unit))
    dY = dist * cos(to_rad(azimuth, angle_unit))

    target_x = base_x + dX
    target_y = base_y + dY
    target_z = base_z + ih + dZ - th

    return dict(x=target_x, y=target_y, z=target_z)


class PolarPoint:
    '''A point geometry defined by polar coordinates.'''

    def __init__(self,
                 angle_unit,          # angle unit
                 z_angle_type,        # type of angle z, v or dh
                 dist_type,           # type of distance h or s
                 dist,                # inclined distance
                 angle,               # horizontal angle
                 z_angle,             # vertical angle
                 th,                  # target height
                 base_point,          # BasePoint object
                 pid,                 # point ID
                 text,                # point description
                 coordorder):   # cartesian coordinates order (NEZ, ENZ)
        self.angle_unit = angle_unit
        self.z_angle_type = z_angle_type
        self.dist_type = dist_type
        self.dist = float(dist)
        self.th = float(th)
        self.angle = float(angle)
        self.z_angle = float(z_angle)
        self.pid = pid
        self.text = text
        self.coordorder = coordorder

        # base point data
        self.base_x = base_point.x
        self.base_y = base_point.y
        # For NEZ coordinate system, an inversion should be done before calculation
        if self.coordorder == "NEZ":
            self.base_x, self.base_y = self.base_y, self.base_x
        self.base_z = base_point.z
        self.ih = base_point.ih
        self.b_zero_st = base_point.b_zero_st

    def to_point(self):
        '''Convert from PolarPoint to (cartesian) Point object'''

        azimuth = (self.b_zero_st + self.angle) % UNITS_CIRCLE[self.angle_unit]

        cart_coords = polar_to_cartesian(self.angle_unit,
                                         self.z_angle_type,
                                         self.dist_type,
                                         self.base_x,
                                         self.base_y,
                                         self.base_z,
                                         self.dist,
                                         azimuth,
                                         self.z_angle,
                                         self.ih,
                                         self.th)

        if self.coordorder == "NEZ":
            cart_coords['x'], cart_coords['y'] = cart_coords['y'], cart_coords['x']

        cart_point = Point(cart_coords['x'],
                           cart_coords['y'],
                           cart_coords['z'])

        return cart_point

    def as_feature(self):
        '''Wrap geometry and other properties like id, description in a Feature.'''

        feature = Feature(self.to_point(),
                          properties={'angle_unit': self.angle_unit},
                          id=self.pid,
                          desc=self.desc)
        return feature


class BasePoint:
    '''A base point to derive cartesian coordinates from polar.

    TODO: find out whether ih is more commonly coupled to a base point or
    to each single point.'''

    def __init__(self, x, y, z, ih, b_zero_st):
        self.x = float(x)
        self.y = float(y)
        self.z = float(z)
        self.ih = float(ih)
        self.b_zero_st = float(b_zero_st)



import importlib

import pytest

from totalopenstation.output import BUILTIN_OUTPUT_FORMATS


class BaseTestOutput:
    '''This base class is inherited by all output test modules.

    pytest will not run the test here because the class name does not
    start with "test".

    Child classes must implement a setup fixture, marked as @pytest.fixture.
    '''

    @pytest.mark.parametrize(
        'output_format', [ of for of in BUILTIN_OUTPUT_FORMATS ]
    )
    def test_output_format(self, output_format, setup):
        tup = BUILTIN_OUTPUT_FORMATS[output_format]
        mod, cls, name = tup
        outputclass = getattr(importlib.import_module('totalopenstation.output.' + mod), cls)
        assert outputclass(self.fp.points).process()



# -*- coding: utf-8 -*-
# filename: formats/carlson_rw5.py
# Copyright 2014 Stefano Costa <steko@iosa.it>
# Copyright 2016 Damien Gaignon <damien.gaignon@gmail.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from totalopenstation.formats.conversion import horizontal_to_slope
from . import Feature, Point, UNKNOWN_STATION, UNKNOWN_POINT
from .polar import BasePoint, PolarPoint

# ussfeet = US Survey Feet
#
UNITS = {"angle": {"0": "dms", "1": "gon"},
         "distance": {"0": "feet", "1": "meter", "2": "ussfeet"}}

logger = logging.getLogger(__name__)

def _record(recstr):
    fields = recstr.split(',')
    record_fields = {f[0:2] : f[2:] for f in fields[1:]}

    # Record type, including comment records
    if len(fields[0]) > 2:
        record_fields['type'] = fields[0].strip('-')
        record_fields['comment'] = True
    else:
        record_fields['type'] = fields[0]

    # Note field
    try:
        record_fields['--']
    except KeyError:
        record_fields['note'] = ''
    else:
        record_fields['note'] = record_fields['--']
    logger.debug("record_fields : %s" % (record_fields))
    return record_fields

class FormatParser:
    '''The FormatParser for Carlson RW5 data format.

    Args:
        data (str): A string representing the file to be parsed.

    Attributes:
        rows (list): A list of each lines of the file being parsed which do not begin with '-- '.
    '''

    def __init__(self, data):
        self.rows = (r for r in data.splitlines() if not r.startswith('-- '))
        # Text comments, but not comment records ------------------------^

    @property
    def points(self):
        '''Extract all RW5 data.

        This parser is based on the information in :ref:`if_carlson_rw5`

        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.

        Raises:

        Notes:
            Sometimes needed records are commented so it is needed to parse also comments
        '''
        points_coord = {}
        base_points = {}
        points = []
        pid = 0

        for row in self.rows:
            rec = _record(row)
            # Get angle and distance units
            if rec['type'] == 'MO':
                angle_unit = UNITS["angle"][rec['AU']]
                dist_unit = UNITS["distance"][rec['UN']]
            # Look for point coordinates
            if rec['type'] == 'SP':
                point_name = rec['PN']
                northing = float(rec['N '])  # extra whitespace
                easting = float(rec['E '])  # extra whitespace
                elevation = float(rec['EL'])
                point = Point(easting, northing, elevation)
                attrib = [rec['note']]
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[point_name] = point
            # Look for station coordinates
            if rec['type'] == 'OC':
                station_name = rec['OP']
                northing = float(rec['N '])  # extra whitespace
                easting = float(rec['E '])  # extra whitespace
                elevation = float(rec['EL'])
                station_point = Point(easting, northing, elevation)
                points_coord[station_name] = station_point
                bp = BasePoint(x=easting, y=northing, z=elevation, ih=0, b_zero_st=0.0)
                base_points[station_name] = bp
            # Look for line of sight values
            # Finalize station computing
            if rec['type'] == 'LS':
                ih = float(rec['HI'])
                th = float(rec['HR'])
                attrib = [rec['note']]
                try:
                    station_point
                except NameError:
                    logger.info('There is no known station')
                    station_point = UNKNOWN_STATION
                stf = Feature(station_point,
                              desc='ST',
                              id=pid,
                              point_name=station_name,
                              dist_unit=dist_unit,
                              attrib=attrib)
                bp = base_points[station_name]
                bp.ih = ih
                # Do not add station if previous station record is the same
                try:
                    last_stf
                except NameError:
                    last_stf = stf
                    points.append(stf)
                    pid += 1
                else:
                    if stf.point_name != last_stf.point_name or \
                                    stf.geometry.x != last_stf.geometry.x or \
                                    stf.geometry.y != last_stf.geometry.y or \
                                    stf.geometry.z != last_stf.geometry.z:
                        points.append(stf)
                        pid += 1
                        last_stf = stf
            # Look for polar data
            if rec['type'] in ('SS', 'TR', 'BD', 'BR', 'FD', 'FR'):
                point_name = rec['FP']
                attrib = [rec['note']]
                # Angle is recorded as azimuth or horizontal angle
                try:
                    angle = float(rec['AZ'])
                except KeyError:
                    # Angle is either Bearing, Angle Right or Left, Deflection Right or Left
                    try:
                        angle = float(rec['BR'])
                    except KeyError:
                        try:
                            angle = float(rec['AR'])
                        except KeyError:
                            try:
                                angle = float(rec['AL'])
                            except KeyError:
                                try:
                                    angle = float(rec['DR'])
                                except KeyError:
                                    try:
                                        angle = float(rec['DL'])
                                    except KeyError:
                                        logger.info('There is no horizontal angle value')
                # Vertical angle is either Zenith, Vertical angle or Change elevation
                try:
                    z_angle = float(rec['ZE'])
                except KeyError:
                    try:
                        z_angle = float(rec['VA'])
                    except KeyError:
                        try:
                            z_angle = float(rec['CE'])
                        except KeyError:
                            logger.info('There is no vertical angle value')
                        else:
                            z_angle_type = 'dh'
                    else:
                        z_angle_type = 'v'
                else:
                    z_angle_type= 'z'
                try:
                    dist = float(rec['SD'])
                except KeyError:
                    try:
                        dist = float(rec['HD'])
                    except KeyError:
                        logger.info('There is no distance value')
                    else:
                        dist_type = "h"
                else:
                    dist_type = 's'
                attrib = [rec['note']]
                p = PolarPoint(angle_unit=angle_unit,
                               z_angle_type=z_angle_type,
                               dist_type=dist_type,
                               dist=dist,
                               angle=angle,
                               z_angle=z_angle,
                               th=th,
                               base_point=bp,
                               pid=pid,
                               text=point_name,
                               coordorder='ENZ')
                point = p.to_point()
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
        logger.debug(points)
        return points

    @property
    def raw_line(self):
        '''Extract all Carlson RW5 data.

        This parser is based on the information in :ref:`if_carlson_rw5`

        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.

        Raises:

        Notes:
            Information needed are:
                - station : Occupy Point and Line of Sight
                - backsight : Backsight
                - direct point : Store Point
                - computed point : Foresight Direct/Reverse, Traverse/Sideshot, Backsight Direct/Reverse

            Sometimes needed records are commented so it is needed to parse also comments
        '''

        points_coord = {}
        points = []
        pid = 0
        station_id = 1

        for row in self.rows:
            rec = _record(row)
            # Get angle and distance units
            if rec['type'] == 'MO':
                angle_unit = UNITS["angle"][rec['AU']]
                dist_unit = UNITS["distance"][rec['UN']]
            # Look for point coordinates
            if rec['type'] == 'SP':
                point_name = rec['PN']
                northing = float(rec['N '])  # extra whitespace
                easting = float(rec['E '])  # extra whitespace
                elevation = float(rec['EL'])
                point = Point(easting, northing, elevation)
                attrib = [rec['note']]
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[point_name] = point
            # Look for station coordinates
            if rec['type'] == 'OC':
                station_name = rec['OP']
                northing = float(rec['N '])  # extra whitespace
                easting = float(rec['E '])  # extra whitespace
                elevation = float(rec['EL'])
                station_point = Point(easting, northing, elevation)
                points_coord[station_name] = station_point
            # Look for line of sight values
            # Finalize station computing
            if rec['type'] == 'LS':
                ih = float(rec['HI'])
                th = float(rec['HR'])
                attrib = [rec['note']]
                try:
                    station_point
                except NameError:
                    logger.info('There is no known station')
                    station_point = UNKNOWN_STATION
                    station_name = 'station_' + str(station_id)
                    points_coord[station_name] = station_point
                stf = Feature(station_point,
                              desc='ST',
                              id=pid,
                              point_name=station_name,
                              angle_unit=angle_unit,
                              dist_unit=dist_unit,
                              ih=ih,
                              attrib=attrib)
                # Do not add station if previous station record is the same
                try:
                    last_stf
                except NameError:
                    last_stf = stf
                    points.append(stf)
                    pid += 1
                else:
                    if stf.point_name != last_stf.point_name or \
                                    stf.geometry.x != last_stf.geometry.x or \
                                    stf.geometry.y != last_stf.geometry.y or \
                                    stf.geometry.z != last_stf.geometry.z or \
                                    stf.properties['ih'] != last_stf.properties['ih']:
                        points.append(stf)
                        pid += 1
                        last_stf = stf
            # Look for back sight values
            if rec['type'] == 'BK':
                point_name = rec['BP']
                circle = rec['BC']
                try:
                    point = points_coord[point_name]
                except KeyError:
                    logger.info('There is no known point')
                    point = UNKNOWN_POINT
                f = Feature(point,
                            desc='BS',
                            id=pid,
                            point_name=point_name,
                            angle_unit=angle_unit,
                            circle=circle)
                points.append(f)
                pid += 1
            # Look for polar data
            if rec['type'] in ('SS', 'TR', 'BD', 'BR', 'FD', 'FR'):
                point_name = rec['FP']
                # Angle is recorded as azimuth or horizontal angle
                try:
                    azimuth = float(rec['AZ'])
                except KeyError:
                    azimuth = None
                # Angle is either Bearing, Angle Right or Left, Deflection Right or Left
                try:
                    angle = float(rec['BR'])
                except KeyError:
                    try:
                        angle = float(rec['AR'])
                    except KeyError:
                        try:
                            angle = float(rec['AL'])
                        except KeyError:
                            try:
                                angle = float(rec['DR'])
                            except KeyError:
                                try:
                                    angle = float(rec['DL'])
                                except KeyError:
                                    logger.info('There is no horizontal angle value')
                                    angle = None
                # Vertical angle is either Zenith, Vertical angle or Change elevation
                try:
                    z_angle = float(rec['ZE'])
                except KeyError:
                    try:
                        z_angle = float(rec['VA'])
                    except KeyError:
                        try:
                            z_angle = float(rec['CE'])
                        except KeyError:
                            logger.info('There is no vertical angle value')
                            z_angle = None
                        else:
                            z_angle_type = 'dh'
                    else:
                        z_angle_type = 'v'
                else:
                    z_angle_type = 'z'
                try:
                    dist = float(rec['SD'])
                except KeyError:
                    try:
                        dist = float(rec['HD'])
                    except KeyError:
                        logger.info('There is no distance value')
                        dist = None
                    else:
                        dist_type = 'h'
                else:
                    dist_type = 's'
                attrib = [rec['note']]
                try:
                    point = points_coord[point_name]
                except KeyError:
                    logger.info('There is no known point')
                    point = UNKNOWN_POINT
                try:
                    station_name
                except UnboundLocalError:
                    logger.info('There is no known station')
                    station_name = 'station_' + str(station_id)
                    station_id += 1
                f = Feature(point,
                            desc='PO',
                            id=pid,
                            point_name=point_name,
                            angle_unit=angle_unit,
                            z_angle_type=z_angle_type,
                            dist_unit=dist_unit,
                            dist_type=dist_type,
                            azimuth=azimuth,
                            angle=angle,
                            z_angle=z_angle,
                            dist=dist,
                            th=th,
                            station_name=station_name,
                            attrib=attrib)
                points.append(f)
                pid += 1

        logger.debug(points)
        return points



import unittest

from totalopenstation.formats import Feature, LineString, Point
from totalopenstation.output.tops_dxf import OutputFormat

class TestCSVOutput(unittest.TestCase):

    def setUp(self):
        self.data = [
            Feature(Point(12.8, 76.3, 56.2),
                    desc='TESTPOINT',
                    id=1),
            Feature(Point(19.8, 26.3, 46.2),
                    desc='TESTPOINT2',
                    id=2),
            Feature(LineString(((17.8, 26.0, 41.2),
                                         (18.8, 26.6, 44.2),
                                         (24.8, 26.9, 42.2))),
                    desc='TESTLINE',
                    id=3),
        ]

    def test_output(self):
        self.output = OutputFormat(self.data, separate_layers=False).process()
        self.assertEqual(self.output.splitlines()[1], 'DXF created from Total Open Station')
        self.assertEqual(self.output.splitlines()[67], 'TESTPOINT')
        self.assertEqual(self.output.splitlines()[103], 'TESTPOINT2')
        self.assertEqual(self.output.splitlines()[139], 'TESTLINE')
        self.assertEqual(self.output.splitlines()[183], 'EOF')



# -*- coding: utf-8 -*-
# filename: formats/nikon_raw_v200.py
# Copyright 2010 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging

from . import Feature, Point, UNITS_CIRCLE, UNKNOWN_STATION, UNKNOWN_POINT, check_coordorder
from .polar import BasePoint, PolarPoint

# ussfeet = US Survey Feet
# Angular Mil is not present as the code is not known
UNITS = {"angle": {"DDDMMSS": "dms", "Gons": "gon", "Degrees": "deg"},
         "distance": {"Feet": "feet", "Metres": "meter", "Feet US": "ussfeet"}
         }

logger = logging.getLogger(__name__)


class FormatParser:
    '''The FormatParser for Nikon Raw v2.00 data.

    Args:
        data (str): A string representing the file to be parsed.

    Attributes:
        rows (list): A list of each lines of the file being parsed.
    '''

    def __init__(self, data):
        self.rows = data.splitlines()

    @property
    def points(self):
        '''Extract all Nikon RAW data format V2.00.

        This parser is based on the information in :ref:`if_nikon_raw`

        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.

        Raises:

        Notes:
            Sometimes needed records are commented so it is needed to parse also comments

            Angles are considered as vertical
            Distances are slope distances
        '''
        points_coord = {}
        base_points = {}
        points = []
        pid = 0
        st = 0
        cocircle = coih = False
        
        for row in self.rows:
            fs = row.split(',')
            # Get angle and distance units
            if fs[0] == 'CO':
                if fs[1].startswith('Coord Order:'):
                    coordorder = check_coordorder(fs[1].split(':')[-1].strip())
                if fs[1].startswith('Angle Units:'):
                    angle_unit = UNITS["angle"][fs[1].split(':')[1].strip()]
                if fs[1].startswith('Dist Units:'):
                    dist_unit = UNITS["distance"][fs[1].split(':')[1].strip()]
                # For fast setting, Trimble M3 can use comments for station
                if fs[1].startswith('TI  HOLD') or fs[1].startswith('TI  Hz'):
                    circle = fs[1].split('Hz')[1].strip().split()[0]
                    cocircle = True
                if fs[1].startswith('TI  INPUT'):
                    try:
                        ih = fs[1].split('ih')[1].strip().split()[0]
                    except IndexError:
                        ih = ih
                    coih = True
                if cocircle and coih:
                    station_name = "st{}".format(st)
                    try:
                        station_point = points_coord[station_name]
                    except KeyError:
                        logger.info('There is no known station')
                        station_point = UNKNOWN_STATION
                        points_coord[station_name] = station_point
                    f = Feature(station_point,
                            desc='ST',
                            id=pid,
                            point_name=station_name,
                            dist_unit=dist_unit,
                            ih=ih)
                    points.append(f)
                    b_zero_st = 0.0
                    bp = BasePoint(x=station_point.x, y=station_point.y, z=station_point.z, ih=ih, b_zero_st=b_zero_st)
                    base_points[station_name] = bp
                    st += 1
                    pid += 1
                    cocircle = coih = False

            # Look for point coordinates
            if fs[0] in ('UP','MP', 'CC', 'RE', 'MC'):
                point_name = fs[1]
                easting = fs[3]
                northing = fs[4]
                if coordorder == "NEZ":
                    easting, northing = northing, easting
                elevation = float(fs[5])
                point = Point(easting, northing, elevation)
                attrib = [fs[6]]
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[point_name] = point
            # Look for station coordinates
            if fs[0] == 'ST':
                station_name = fs[1]
                ih = float(fs[5])
                if ih == '':
                    ih = 0
                try:
                    station_point = points_coord[station_name]
                except KeyError:
                    logger.info('There is no known station')
                    station_point = UNKNOWN_STATION
                    points_coord[station_name] = station_point
                # Look for back sight values in station values
                # Treat only one backsight or the last one
                if fs[3] != '':
                    b_zero_st = (float(fs[6]) - float(fs[7])) % UNITS_CIRCLE[angle_unit]
                else:
                    b_zero_st = 0.0
                f = Feature(station_point,
                            desc='ST',
                            id=pid,
                            point_name=station_name,
                            dist_unit=dist_unit,
                            ih=ih,
                            b_zero_st=b_zero_st)
                points.append(f)
                pid += 1
                bp = BasePoint(x=station_point.x, y=station_point.y, z=station_point.z, ih=ih, b_zero_st=b_zero_st)
                base_points[station_name] = bp
            # Look for Sideshot, Face 1 and Face 2
            if fs[0] in ('SS', 'F1', 'F2'):
                for i in (2, 3, 4, 5):
                    if fs[i] == '':
                        fs[i] = 0
                point_name = fs[1]
                th = float(fs[2])
                dist = float(fs[3])
                angle = float(fs[4])
                z_angle = float(fs[5])
                try:
                    attrib = [fs[7]]
                except IndexError:
                    attrib = []
                p = PolarPoint(angle_unit=angle_unit,
                               z_angle_type='v',
                               dist_type='s',
                               dist=dist,
                               angle=angle,
                               z_angle=z_angle,
                               th=th,
                               base_point=bp,
                               pid=pid,
                               text=point_name,
                               coordorder=coordorder)
                point = p.to_point()
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[point_name] = point
            # Look for Stakeout
            if fs[0] == 'SO':
                point_name = fs[1]
                th = float(fs[3])
                dist = float(fs[4])
                angle = float(fs[5])
                z_angle = float(fs[6])
                p = PolarPoint(angle_unit=angle_unit,
                               z_angle_type='v',
                               dist_type='s',
                               dist=dist,
                               angle=angle,
                               z_angle=z_angle,
                               th=th,
                               base_point=bp,
                               pid=pid,
                               text=point_name,
                               coordorder=coordorder)
                point = p.to_point()
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit)
                points.append(f)
                pid += 1
                points_coord[station_name] = point
            # Look for Control point
            if fs[0] == 'CP':
                point_name = fs[1]
                th = float(fs[3])
                dist = float(fs[4])
                angle = float(fs[5])
                z_angle = float(fs[6])
                attrib = [fs[8]]
                p = PolarPoint(angle_unit=angle_unit,
                               z_angle_type='v',
                               dist_type='s',
                               dist=dist,
                               angle=angle,
                               z_angle=z_angle,
                               th=th,
                               base_point=bp,
                               pid=pid,
                               text=point_name,
                               coordorder=coordorder)
                point = p.to_point()
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[station_name] = point
        logger.debug(points)
        return points

    @property
    def raw_line(self):
        '''Extract all Nikon Raw v2.00 data.

        This parser is based on the information in :ref:`if_nikon_raw`

        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.

        Raises:

        Notes:
            Information needed are:
                - station : Station record
                - backsight : Control point record
                - direct point : Uploaded point or Manually input point or Calculated coordinate or Resection point
                - computed point : Sideshot, Stakeout, Face 1 or 2
  		  
           Sometimes needed records are commented so it is needed to parse also comments like
                - coordinates order
                - units

           Angles are considered as vertical
           Distances are slope distances
        '''
        points_coord = {}
        points = []
        pid = 0
        st = 0
        cocircle = coih = False

        for row in self.rows:
            fs = row.split(',')
            # Get angle and distance units
            if fs[0] == 'CO':
                if fs[1].startswith('Coord Order:'):
                    coordorder = check_coordorder(fs[1].split(':')[-1].strip())
                if fs[1].startswith('Angle Units:'):
                    angle_unit = UNITS["angle"][fs[1].split(':')[1].strip()]
                if fs[1].startswith('Dist Units:'):
                    dist_unit = UNITS["distance"][fs[1].split(':')[1].strip()]
                # For fast setting, Trimble M3 can use comments for station
                if fs[1].startswith('TI  HOLD') or fs[1].startswith('TI  Hz'):
                    circle = fs[1].split('Hz')[1].strip().split()[0]
                    cocircle = True
                if fs[1].startswith('TI  INPUT'):
                    try:
                        ih = fs[1].split('ih')[1].strip().split()[0]
                    except IndexError:
                        ih = ih
                    coih = True
            if cocircle and coih:
                station_name = "st{}".format(st)
                try:
                    station_point = points_coord[station_name]
                except KeyError:
                    logger.info('There is no known station')
                    station_point = UNKNOWN_STATION
                    points_coord[station_name] = station_point
                f = Feature(station_point,
                        desc='ST',
                        id=pid,
                        point_name=station_name,
                        dist_unit=dist_unit,
                        ih=ih)
                points.append(f)
                st += 1
                pid += 1
                cocircle = coih = False

            # Look for point coordinates
            if fs[0] in ('UP','MP', 'CC', 'RE', 'MC'):
                point_name = fs[1]
                easting = fs[3]
                northing = fs[4]
                if coordorder == "NEZ":
                    easting, northing = northing, easting
                elevation = float(fs[5])
                point = Point(easting, northing, elevation)
                attrib = [fs[6]]
                f = Feature(point,
                            desc='PT',
                            id=pid,
                            point_name=point_name,
                            dist_unit=dist_unit,
                            attrib=attrib)
                points.append(f)
                pid += 1
                points_coord[point_name] = point
            # Look for station coordinates
            if fs[0] == 'ST':
                station_name = fs[1]
                ih = float(fs[5])
                if ih == '':
                    ih = 0
                try:
                    station_point = points_coord[station_name]
                except KeyError:
                    logger.info('There is no known station')
                    station_point = UNKNOWN_STATION
                    points_coord[station_name] = station_point
                f = Feature(station_point,
                            desc='ST',
                            id=pid,
                            point_name=station_name,
                            dist_unit=dist_unit,
                            ih=ih)
                points.append(f)
                pid += 1
                # Look for back sight values in station values
                # Treat only one backsight or the last one
                if fs[3] != '':
                    b_zero_st = (float(fs[6]) - float(fs[7])) % UNITS_CIRCLE[angle_unit]
                else:
                    b_zero_st = 0.0
                if fs[3] != '':
                    point_name = fs[3]
                    azimuth = fs[6]
                    circle = fs[7]
                    try:
                        point = points_coord[point_name]
                    except KeyError:
                        logger.info('There is no known point')
                        point = UNKNOWN_POINT
                    f = Feature(point,
                                desc='BS',
                                id=pid,
                                point_name=point_name,
                                angle_unit=angle_unit,
                                circle=circle,
                                azimuth=azimuth)
                    points.append(f)
                    pid += 1
            # Look for Sideshot, Face 1 and Face 2
            if fs[0] in ('SS', 'F1', 'F2'):
                for i in (2, 3, 4, 5):
                    if fs[i] == '':
                        fs[i] = 0
                point_name = fs[1]
                th = float(fs[2])
                dist = float(fs[3])
                angle = float(fs[4])
                z_angle = float(fs[5])
                try:
                    attrib = [fs[7]]
                except IndexError:
                    attrib = []
                azimuth = None
                try:
                    point = points_coord[point_name]
                except KeyError:
                    logger.info('There is no known point')
                    point = UNKNOWN_POINT
                f = Feature(point,
                            desc='PO',
                            id=pid,
                            point_name=point_name,
                            angle_unit=angle_unit,
                            z_angle_type='v',
                            dist_unit=dist_unit,
                            dist_type='s',
                            azimuth=azimuth,
                            angle=angle,
                            z_angle=z_angle,
                            dist=dist,
                            th=th,
                            attrib=attrib)
                points.append(f)
                pid += 1
            # Look for Stakeout
            if fs[0] == 'SO':
                point_name = fs[1]
                th = float(fs[3])
                dist = float(fs[4])
                angle = float(fs[5])
                z_angle = float(fs[6])
                azimuth = None
                try:
                    point = points_coord[point_name]
                except KeyError:
                    logger.info('There is no known point')
                    point = UNKNOWN_POINT
                f = Feature(point,
                            desc='PO',
                            id=pid,
                            point_name=point_name,
                            angle_unit=angle_unit,
                            z_angle_type='v',
                            dist_unit=dist_unit,
                            dist_type='s',
                            azimuth=azimuth,
                            angle=angle,
                            z_angle=z_angle,
                            dist=dist,
                            th=th)
                points.append(f)
                pid += 1
            # Look for Control point
            if fs[0] == 'CP':
                point_name = fs[1]
                th = float(fs[3])
                dist = float(fs[4])
                angle = float(fs[5])
                z_angle = float(fs[6])
                attrib = fs[8]
                azimuth = None
                try:
                    point = points_coord[point_name]
                except KeyError:
                    logger.info('There is no known point')
                    point = UNKNOWN_POINT
                f = Feature(point,
                            desc='PO',
                            id=pid,
                            point_name=point_name,
                            angle_unit=angle_unit,
                            z_angle_type='v',
                            dist_unit=dist_unit,
                            dist_type='s',
                            azimuth=azimuth,
                            angle=angle,
                            z_angle=z_angle,
                            dist=dist,
                            th=th,
                            attrib=attrib)
                points.append(f)
                pid += 1
        logger.debug(points)
        return points



# -*- coding: utf-8 -*-
# filename: formats/sokkia_sdr33.py
# Copyright 2014 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Feature, Parser, Point


class FormatParser(Parser):

    def is_point(self, line):
        if line[2:4] == ('TP'):
            return True
        else:
            return False

    def get_point(self, line):
        id = int(line[12:20])
        y = float(line[20:32])  # Northing
        x = float(line[32:48])  # Easting
        z = float(line[48:63])  # Elevation

        if line[0:2] == '02':   # Base point
            desc = line[78:86].strip()
        if line[0:2] == '08':   # Measurement
            desc = line[63:70].strip()

        point = Point(x, y, z)
        feature = Feature(point, desc=desc, id=id)
        return feature



import json
import unittest

from totalopenstation.formats import Feature, FeatureCollection, Point
from totalopenstation.output.tops_geojson import OutputFormat

class TestGeoJSONOutput(unittest.TestCase):

    def setUp(self):
        self.data = [
            Feature(Point(12.8, 76.3, 56.2),
                    desc='TEST POINT',
                    id=1),
            Feature(Point(19.8, 26.3, 46.2),
                    desc='TEST POINT #2',
                    id=2),
        ]

    def test_output(self):
        self.output = OutputFormat(self.data).process()
        ref_output = '''{"type": "FeatureCollection", "bbox": [12.8, 26.3, 56.2, 19.8], "features": [{"bbox": [12.8, 76.3, 56.2, 12.8, 76.3, 56.2], "geometry": {"type": "Point", "coordinates": [12.8, 76.3, 56.2]}, "type": "Feature", "properties": {"desc": "TEST POINT"}, "id": 1}, {"bbox": [19.8, 26.3, 46.2, 19.8, 26.3, 46.2], "geometry": {"type": "Point", "coordinates": [19.8, 26.3, 46.2]}, "type": "Feature", "properties": {"desc": "TEST POINT #2"}, "id": 2}]}'''
        self.assertEqual(json.loads(self.output), json.loads(ref_output))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: totalopenstation-gui.py
# Copyright 2008-2019 Stefano Costa <steko@iosa.it>
# Copyright 2010,2012 Luca Bianconi <luxetluc@yahoo.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import serial
import gettext
import atexit

from time import sleep

from tkinter import *
from tkinter.messagebox import showwarning, showinfo, askokcancel
import tkinter.simpledialog
import tkinter.filedialog

import totalopenstation

from totalopenstation.models import BUILTIN_MODELS
from totalopenstation.formats import BUILTIN_INPUT_FORMATS
from totalopenstation.output import BUILTIN_OUTPUT_FORMATS
from totalopenstation.utils.upref import UserPrefs

t = gettext.translation('totalopenstation', './locale', fallback=True)
_ = t.gettext


def scan():
    """scan for available ports. return a list of tuples (num, name).

    Part of pySerial (http://pyserial.sf.net) Copytight 2002-2003 <cliechti@gmx.net>
    """

    # TODO move this function in a separate module together with scanwin32.py
    # and add conditional loading depending on the operating system

    available = []
    for i in range(256):
        try:
            s = serial.Serial(i)
            available.append((i, s.portstr))
            s.close()   #explicit close 'cause of delayed GC in java
        except serial.SerialException:
            pass
    return available

# logo GIF image encoded as base64 string
# this way we don't need external an external image file
logo_data = '''
R0lGODlhMAAwAOfIAC40NjA2NzY6ODw+O0g8MEBBP09JOVRMO1hPP0NDQk9MQVRWVFhaV1tcW2Rn\n
ZGZpZm1ubHFzbnV3c3h7dnx+ebw/P8wAAM8NDdEGBtQLC9gNDcUcHM4ZGdMTE98XF9MdHd8ZGc8q\n
KtM3N9Y7O+8pKeM7O/A0NJ1QEbV6MM5cAMFeDdBfBcJfEcVgDc1lDtFgA9xqA9BjCtVoCcNhE8hm\n
FMVlGtRvHdVyFdJzHeVuAexzAPJ2APV5AfV+CsduJspzLtJ0IMx3M8t4Nd5aWuRJSPFERPFLS+Nc\n
XNZ7W990dOJjY+hnZ+Jsa/FgYPFsbON0dPV2dvR5eX6AfL2JPPaDDPaFE/eKFPaGGfeNG/iSHd6B\n
Jd6DKd2FMtyEPMmRPOmNKPeMJO+XLfeSJPiWIfmZJfWULPmbKumQMOmVPveVNfKcM/mfMfmfOvqg\n
LfqkMvmlO/yuPYCCfc+ESNCFSMmZT9WSXPGgRPerQvuuQvGiSPerSvquSfywQ/yySfemVfinV/ev\n
U/qsUfuzVPy4VPq1WPy6Xdqhc9yoffmvYvmxZfy+Zfq1avu8beereuiufvu7c/3CbPzDdPzEe/3J\n
fYOFgYeIg4qNh46RjJCUkLGOjqmsprW1tb66ut+ehe2Hh+2Li/aWlveYmN+wi+Cvh+CwieK3leW9\n
nPrHgP3Lgv3Oif3Skv3WnenHrenIrv7apvXTrf7fsfLTuv7gtv7iuMHBwcrKytra2u7Vwunazeve\n
0/Lf0O3h1+3h2PPg0v7t1Pbp3+Xl5ezs7PHl5fHr6/vs7Pnx6vPz8/z38///////////////////\n
////////////////////////////////////////////////////////////////////////////\n
////////////////////////////////////////////////////////////////////////////\n
/////////////////////////////////////////////////////yH5BAEAAP8ALAAAAAAwADAA\n
AAj+AP8JHEiwoMBYNlIoXJjCRiyDECNKhGjjla+LFx/98WNjoseP/xotlKWqZEkeORim6AKy5cAU\n
s1ylUKWoZk0eM1OhiqQohUuXMOGkiASnaFGcqo6m8fkTZNATCgIEMDAFDs5UR8swbeoRppcEEyhR\n
cDDAC05UR8Fs5SoRJgJKDBIssIQJAc5JadeyhQhTQKUEmxpUuiTg7tErevcWhHlgwgIGEKQ8OICT\n
qNUqiRW/nOWlgARKjgfQ0QrpaI/Mmv8FRUFAKoETC0tbxZma76yjtF8qwo1ac9DZKl8U4l3b4G81\n
YZIrV2OUR+7iA1fImgXL1SpVqSZFglRoUB8+zl/+QCfoSKV5lY7GRwTAII4mTZciLACgfuIRAPjz\n
51dSH+ITT05gYMGAA2LQxBFP9EcQMR+AUgIUEDZhhAkklBBFB8QoKNATn4SiARQkhCiiBlEskaCC\n
DIJihAUgihiiBUVcmGF/TxARCgksuvgiCVGUcGJ9TxCYo45CMqEgKVwwwgOLRhDJAyNckNJfMSxE\n
gsWSFgxB4AUD8oBFJCwUU18daATCgw4zpIAIFWxakcIMOvAQCBp1qNcLDZFQwYMLpqSQCJtUYJGC\n
KS7wQAUjNPQynhB5sIGSD6otYsWkgv7jQw48sGGHENC1AoQizrVwS6STWiGGT7e04FwhOLRSXA3+\n
gIjBAwxzCJTCIljkeqpAc8DAgxh61FDbKFsIwsMOKvxiKyNZNGsGU7+osAMPe2gximbFzEBIFTzI\n
YMhLj4wh7rMDGSIDD1XsMYOYe9VxxhtnzkBQCo+QYe8aa8HJwxpf1MmWLjQIoief80ZixsFurEUo\n
Dz3gQYMubAWhhhmPLiZJGxi/odelPIxRRhBcsYJDH6GOOq8kbqSscUGpOufGDaw0VcMbV8Igh3Eo\n
pyyUQXL4aoUZwrpUyg1uxMvLQMEkncIpbzSNRwpJBzPQLvqacYMoLnVCoBC5/BN10sCkgFZRfKQA\n
zNdS4/IDgUi49AEIHSQh0NfA1C22UX2YXfdx1wKNkIEHFrQUjAUZiDDM3FHXHfYpqHB3h95nRy2Q\n
MB0IKLVHwNBiQQWc2IJ04sAcEsNCMRyyt+QC2cLJBhbQAsxHwdhiy+sEoZ20BTjafvlAwMi+e23B\n
ABNCBhycraFEwdSSiQWZ1PL78Qb1TrtiAQEAOw==\n'''

class StatusBar(Frame):
    '''A status bar for Tkinter applications.

    From: http://effbot.org/tkinterbook/tkinter-application-windows.htm
    '''

    def __init__(self, master):
        Frame.__init__(self, master)
        self.label = Label(self, bd=1, relief=SUNKEN, anchor=W)
        self.label.pack(fill=X)

    def set(self, format, *args):
        self.label.config(text=format % args)
        self.label.update_idletasks()

    def clear(self):
        self.label.config(text="")
        self.label.update_idletasks()

class AboutDialog(tkinter.simpledialog.Dialog):

    def body(self, master):
        title = "Total Open Station %s" % totalopenstation.__version__
        message = _("""
Total Open Station is copyright 2008-2019 Stefano Costa, Damien
Gaignon, Luca Bianconi and the IOSA project, under the GNU GPL v3
or any later version.

https://tops.iosa.it/

The application logo is copyright 2008 Lapo Calamandrei under the same
license.""")

        self.logo_data = logo_data
        self.logo = PhotoImage(data=self.logo_data)
        Label(master, image=self.logo).pack()

        Label(master, text=title, font=("Helvetica", "16", "bold")).pack()
        Label(master, text=message).pack()

    def buttonbox(self):
        box = Frame(self)
        w = Button(box,
                   text="OK",
                   width=10,
                   command=self.cancel,
                   default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        self.bind("&lt;Return>", self.cancel)
        self.bind("&lt;Escape>", self.cancel)
        box.pack()


class DownloadDialog(tkinter.simpledialog.Dialog):

    def body(self, master):
        title = "Total Open Station download"
        message = _("""
Press OK when you're ready to download.\n
Depending on your device, you may need to start the transfer from\n
the total station menu.""")

        Label(master, text=title, font=("Helvetica", "16", "bold")).pack()
        self.msg_var = StringVar()
        self.msg_var.set(message)
        self.msg = Label(master, textvariable=self.msg_var)
        self.msg.pack()

    def apply(self):
        self.result = True


class ConnectDialog(tkinter.simpledialog.Dialog):

    def __init__(self, parent, cs):
        self.conn_str = cs
        tkinter.simpledialog.Dialog.__init__(self, parent)

    def body(self, master):
        title = _("waiting for data from device")
        message1 = _("Connection initialized with the following parameters:\n")
        params = "%s\n" % self.conn_str
        message2 = _("\nStart the download procedure on the device.\n")
        message2 = message2 + _("Press OK when done.")
        Label(master, bitmap="hourglass").pack()
        Label(master, text=title, font=("Helvetica", "16", "bold")).pack()
        Label(master, text=message1).pack()
        t = Text(master,
             width=80)
        t.insert(END, params)
        t.pack()
        Label(master, text=message2, fg="red").pack()


class ProcessDialog(tkinter.simpledialog.Dialog):

    def __init__(self, parent, data):
        self.data = data
        self.format = ''
        tkinter.simpledialog.Dialog.__init__(self, parent)

    def body(self, master):
        title = _("Choose output format and destination file")
        question = _("Output format:\n")
        top_frame = Frame(master)
        top_frame.pack(side=TOP, padx=5, pady=5)

        control_panel0 = Frame(master)
        control_panel0.pack(
            side=TOP,
            expand=YES,
            anchor=S,
            fill=Y, padx=5, pady=5)
        bottom_frame = Frame(master)
        bottom_frame.pack(side=TOP, anchor=S)

        Label(top_frame, bitmap="question").pack(side=TOP, anchor=N)
        Label(top_frame, text=title).pack(side=TOP, anchor=N)
        output_frame = Frame(control_panel0, relief=RIDGE, bd=1)
        input_frame = Frame(control_panel0, relief=RIDGE, bd=1)
        input_frame.pack(side=TOP)
        output_frame.pack(side=TOP)

        params = "%s\n" % self.data

        Label(input_frame, text=_('Input format'), width=20).pack(side=LEFT)

        self.input_format = StringVar()
        self.input_format.set(self.format)
        input_format_entry = Menubutton(input_frame,
                                        text=_("choose a format"),
                                        textvariable=self.input_format,
                                        relief=RAISED,
                                        width=24)
        input_format_entry.menu = Menu(input_format_entry, tearoff=0)
        input_format_entry["menu"] = input_format_entry.menu

        for k, v in sorted(BUILTIN_INPUT_FORMATS.items()):
            input_format_entry.menu.add_radiobutton(
                label=v[2],
                variable=self.input_format,
                value=k)
        input_format_entry.pack(side=LEFT, anchor=W)

        Label(output_frame, text=_('Output format'), width=20).pack(side=LEFT)

        self.output_format = StringVar()
        self.output_format.set(self.format)
        output_format_entry = Menubutton(output_frame,
                                        text=_("choose a format"),
                                        textvariable=self.output_format,
                                        relief=RAISED,
                                        width=24)
        output_format_entry.menu = Menu(output_format_entry, tearoff=0)
        output_format_entry["menu"] = output_format_entry.menu

        for k, v in sorted(BUILTIN_OUTPUT_FORMATS.items()):
            output_format_entry.menu.add_radiobutton(
                label=v[2],
                variable=self.output_format,
                value=k)
        output_format_entry.pack(side=LEFT, anchor=W)

    def validate(self):
        # do nothing if input is empty
        if self.output_format.get() == '' or self.input_format.get() == '':
            return False
        else:
            return True

    def apply(self):
        '''Export data in the required output format'''

        inputclass = BUILTIN_INPUT_FORMATS[self.input_format.get()]

        # import input format parser
        if isinstance(inputclass, tuple):
            try:
                # builtin format parser
                mod, cls, name = inputclass
                inputclass = getattr(
                    __import__('totalopenstation.formats.' + mod, None, None, [cls]), cls)
            except ImportError as msg:
                showwarning(_('Import error'),
                            _('Error loading the required input module: %s' % msg))

        # import output format writer
        of_lower = str(self.output_format.get()).lower()
        outputclass = BUILTIN_OUTPUT_FORMATS[self.output_format.get()]
        if isinstance(outputclass, tuple):
            try:
                # builtin output builder
                mod, cls, name = outputclass
                outputclass = getattr(
                    __import__('totalopenstation.output.' + mod, None, None, [cls]), cls)
            except ImportError as msg:
                showwarning(_('Import error'),
                            _('Error loading the required output module: %s' % msg))

        # no point in parsing before the output format has been imported
        parsed_data = inputclass(self.data)
        parsed_points = parsed_data.points
        output = outputclass(parsed_points)
        sd = tkinter.filedialog.asksaveasfilename(defaultextension='.%s' % of_lower)

        try:
            sd_file = open(sd, 'w')
        except TypeError:
            showwarning(_("No output file specified"),
                        _("No processing settings entered!\n"))
        else:
            sd_file.write(output.process())
            sd_file.close()

class PreferencesDialog(tkinter.simpledialog.Dialog):
    '''A dialog to change preferences and options.'''



class ErrorDialog(tkinter.simpledialog.Dialog):

    def __init__(self, parent, message):
        self.message = message
        tkinter.simpledialog.Dialog.__init__(self, parent)

    def body(self, master):
        title = _("Error")
        message1 = _("Connection failed with the following error message:\n")
        message2 = _("\nCheck your connection parameters and try again.\n")
        Label(master, bitmap="error", fg="red").pack()
        Label(master, text=title, font=("Helvetica", "16", "bold")).pack()
        Label(master, text=message1).pack()
        t = Entry(master, width=80)
        t.insert(END, self.message)
        t.pack()
        Label(master, text=message2).pack()

    def buttonbox(self):
        box = Frame(self)
        w = Button(box,
                   text=_("Cancel"),
                   width=10,
                   command=self.cancel,
                   default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        self.bind("&lt;Return>", self.cancel)
        self.bind("&lt;Escape>", self.cancel)
        box.pack()


class Tops:

    def __init__(self, parent):

        #init user's preferences config file object
        self.upref = UserPrefs()

        buttons_width = 8
        imb_buttonx = "2m"
        imb_buttony = "1m"
        imb_buttons_framex = "3m"
        imb_buttons_framey = "2m"
        imb_int_buttons_framex = "3m"
        imb_int_buttons_framey = "1m"

        self.myParent = parent

        self.main_frame = Frame(parent) ###
        self.main_frame.pack(expand=YES, fill=BOTH)

		#MENU
        self.menubar = Menu(self.myParent)
        self.myParent.config(menu=self.menubar)

        topsmenu = Menu(self.menubar, tearoff=0)
        topsmenu.add_command(label=_("Connect"), command=self.connect)
        topsmenu.add_command(label=_("Process data"), command=self.process)
        topsmenu.add_separator()
        topsmenu.add_command(label=_("Quit"), command=self.on_app_close)

        filemenu = Menu(self.menubar, tearoff=0)
        filemenu.add_command(label=_("Open file"), command=self.open_a_file)
        filemenu.add_command(label=_("Save raw data"), command=self.save_a_file)

        helpmenu = Menu(self.menubar, tearoff=0)
        helpmenu.add_command(label=_("About TOPS"), command=self.about)

        languagemenu = Menu(self.menubar, tearoff=0)
        language = StringVar()
        languagemenu.add_radiobutton(label="English",variable=language,value="en")
        languagemenu.add_radiobutton(label="Italiano",variable=language,value="it")

        self.menubar.add_cascade(label="Total Station", menu=topsmenu)
        self.menubar.add_cascade(label="File", menu=filemenu)
        #self.menubar.add_cascade(label="Language", menu=languagemenu)
        self.menubar.add_cascade(label="Help", menu=helpmenu)

        self.upper_frame = Frame(self.main_frame) ###
        self.upper_frame.pack(side=TOP, expand=NO, padx=10,
                                   pady=5, ipadx=5, ipady=5)

        self.logo_frame = Frame(self.upper_frame)
        self.logo_frame.pack(side=LEFT, expand=NO)

        self.logo_data = logo_data
        self.logo = PhotoImage(data=self.logo_data)
        self.logo_canvas = Label(self.logo_frame, image=self.logo)
        self.logo_canvas.pack(side=LEFT, expand=NO, padx=5,
                                   pady=5)

        self.header_frame = Frame(self.upper_frame)
        self.header_frame.pack(side=LEFT, expand=NO, pady=5)

        self.buttons_frame = Frame(self.header_frame)
        self.buttons_frame.pack(
            side=TOP,
            expand=NO,
            fill=Y,
            ipadx=5,
            ipady=5)

        # default control panel
        self.control_panel0 = Frame(self.header_frame)
        self.control_panel0.pack(
            side=TOP,
            expand=YES,
            fill=Y, padx=5, pady=5)

        # control panel for custom serial connection
        self.control_panel = Frame(self.header_frame)

        # option 1 : serial port
        self.option1_frame = Frame(self.control_panel0, relief=RIDGE, bd=1)
        self.option1_frame.pack(side=TOP)

        self.option1_label = Label(self.option1_frame,
                                   text=_("Port"),
                                   width=30)
        self.option1_label.pack(side=LEFT)
        self.option1_value = StringVar()
        self.option1_value.set(self.upref.getvalue('port'))

# Leave this Entry uncommented to enter port as a string, or ...
#
        self.option1_entry = Entry(self.option1_frame,
                                   textvariable=self.option1_value,
                                   width=20)

# ... comment out this Menubutton if you want to use the scan() output
#
#        self.option1_entry = Menubutton(self.option1_frame,
#                                        text="choose a value",
#                                        textvariable=self.option1_value,
#                                        relief=RAISED,
#                                        width=24)
#        self.option1_entry.menu = Menu( self.option1_entry, tearoff=0 )
#        self.option1_entry["menu"] = self.option1_entry.menu
#        for n,s in scan():
#            self.option1_entry.menu.add_radiobutton ( label=s,
#                                           variable=self.option1_value,
#                                           value = s)

        self.option1_entry.pack(side=LEFT, anchor=W)

        # option MODEL substitutes all connection parameters for better
        # user experience

        self.optionMODEL_frame = Frame(
            self.control_panel0,
            relief=RIDGE,
            bd=1)
        self.optionMODEL_frame.pack(side=TOP)

        self.optionMODEL_label = Label(self.optionMODEL_frame,
                                   text=_("Total Station"),
                                   justify=LEFT,
                                   width=30)
        self.optionMODEL_label.pack(side=LEFT, anchor=E)
        self.optionMODEL_value = StringVar()

        self.optionMODEL_value.set(self.upref.getvalue('model'))

        self.optionMODEL_entry = Menubutton(self.optionMODEL_frame,
                                        text="choose a model",
                                        textvariable=self.optionMODEL_value,
                                        relief=RAISED,
                                        width=20)
        self.optionMODEL_entry.menu = Menu(self.optionMODEL_entry, tearoff=0)
        self.optionMODEL_entry["menu"] = self.optionMODEL_entry.menu

        for k, (l, m, n) in list(BUILTIN_MODELS.items()):
            self.optionMODEL_entry.menu.add_radiobutton(
                label=n,
                variable=self.optionMODEL_value,
                value=k,
                command=self.print_model)
        self.optionMODEL_entry.pack(side=LEFT, anchor=W)

        # option 2 : baudrate
        self.option2_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
        self.option2_frame.pack(side=TOP)

        self.option2_label = Label(self.option2_frame,
                                   text="Baudrate",
                                   width=30)
        self.option2_label.pack(side=LEFT)
        self.option2_value = IntVar()

        try:
            assert serial.baudrate_constants
            assert serial.baudrate_constants is not {}
        except (AttributeError, AssertionError):
            self.option2_entry = Entry(self.option2_frame,
                                       textvariable=self.option2_value,
                                       width=20)
        else:
            self.option2_entry = Menubutton(self.option2_frame,
                                            text=_("choose a value"),
                                            textvariable=self.option2_value,
                                            relief=RAISED,
                                            width=20)
            self.option2_entry.menu = Menu(self.option2_entry, tearoff=0)
            self.option2_entry["menu"] = self.option2_entry.menu
            for key in sorted(serial.baudrate_constants.keys()): # dynamic list
                self.option2_entry.menu.add_radiobutton(
                    label="%s" % key,
                    variable=self.option2_value,
                    value=key,
                    )
        self.option2_entry.pack(side=LEFT, anchor=W)

        # option 3 : bytesize
        self.option3_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
        self.option3_frame.pack(side=TOP)

        self.option3_label = Label(self.option3_frame,
                                   text=_("Bytesize"),
                                   justify=LEFT,
                                   width=30)
        self.option3_label.pack(side=LEFT, anchor=E)
        self.option3_value = IntVar()
        self.option3_entry = Menubutton(self.option3_frame,
                                        text=_("choose a value"),
                                        textvariable=self.option3_value,
                                        relief=RAISED,
                                        width=20)
        self.option3_entry.menu = Menu(self.option3_entry, tearoff=0)
        self.option3_entry["menu"] = self.option3_entry.menu
        for v in [8,7,6,5]:
            self.option3_entry.menu.add_radiobutton(label=str(v),
                                                    variable=self.option3_value,
                                                    value=v)
        self.option3_entry.pack(side=LEFT, anchor=W)

        # option 4 : parity
        self.option4_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
        self.option4_frame.pack(side=TOP)

        self.option4_label = Label(self.option4_frame,
                                   text=_("Parity setting"),
                                   justify=LEFT,
                                   width=30)
        self.option4_label.pack(side=LEFT, anchor=E)
        self.option4_value = StringVar()
        self.option4_entry = Menubutton(self.option4_frame,
                                        text=_("choose a value"),
                                        textvariable=self.option4_value,
                                        relief=RAISED,
                                        width=20)
        self.option4_entry.menu = Menu(self.option4_entry, tearoff=0)
        self.option4_entry["menu"] = self.option4_entry.menu
        for v in ['Even', 'None', 'Odd']:
            self.option4_entry.menu.add_radiobutton(label=_(v),
                                                    variable=self.option4_value,
                                                    value=v[0])
        self.option4_entry.pack(side=LEFT, anchor=W)

        # option 5 : stop bit
        self.option5_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
        self.option5_frame.pack(side=TOP)

        self.option5_label = Label(self.option5_frame,
                                   text=_("Stop bit"),
                                   justify=LEFT,
                                   width=30)
        self.option5_label.pack(side=LEFT, anchor=E)
        self.option5_value = IntVar()
        self.option5_entry = Menubutton(self.option5_frame,
                                        text=_("choose a value"),
                                        textvariable=self.option5_value,
                                        relief=RAISED,
                                        width=20)
        self.option5_entry.menu = Menu(self.option5_entry, tearoff=0)
        self.option5_entry["menu"] = self.option5_entry.menu
        for v in [1, 2]:
            self.option5_entry.menu.add_radiobutton(label=str(v),
                                                    variable=self.option5_value,
                                                    value=v)
        self.option5_entry.pack(side=LEFT, anchor=W)

        # option 6 : time lapse between data packets
        self.option6_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
        self.option6_frame.pack(side=TOP)

        self.option6_label = Label(self.option6_frame,
                                   text=_("Time lapse between data packets"),
                                   justify=LEFT,
                                   width=30)
        self.option6_label.pack(side=LEFT, anchor=E)
        self.option6_value = DoubleVar()
        self.option6_value.set(self.upref.getvalue('sleeptime'))
        self.option6_entry = Entry(self.option6_frame,
                                   textvariable=self.option6_value,
                                   relief=RAISED,
                                   justify=RIGHT,
                                   width=20)
        self.option6_entry.pack(side=LEFT, anchor=W)

        # control buttons

        self.connect_button = Button(self.buttons_frame,
                                      text=_("Connect"),
                                      padx=imb_buttonx,
                                      pady=imb_buttony)
        self.connect_button.pack(side=LEFT, anchor=S)
        self.connect_button.bind("<Button-1>", self.connect_action)
        self.connect_button.bind("<Return>", self.connect_action)

        self.save_button = Button(self.buttons_frame,
                                      text=_("Save raw data"),
                                      padx=imb_buttonx,
                                      pady=imb_buttony)
        self.save_button.pack(side=LEFT, anchor=S)
        self.save_button.bind("<Button-1>", self.save_action)
        self.save_button.bind("<Return>", self.save_action)

        self.process_button = Button(self.buttons_frame,
                                     text=_("Process data"),
                                     padx=imb_buttonx,
                                     pady=imb_buttony)
        self.process_button.pack(side=LEFT, anchor=S)
        self.process_button.bind("<Button-1>", self.process_action)
        self.process_button.bind("<Return>", self.process_action)

        self.status = StatusBar(self.main_frame)
        self.status.set('Welcome to Total Open Station')
        self.status.pack(side=BOTTOM, fill=X)

        # text frame
        self.text_frame = Frame(self.main_frame)
        self.text_frame.pack(side=BOTTOM, expand=YES, fill=BOTH)

        self.text_area = Text(self.text_frame, width=80)
        self.text_area.insert(END, _("Welcome.\nTurn your device on."))
        self.text_area.pack(side=LEFT, expand=YES, fill=Y)

        self.scrollY = Scrollbar(self.text_frame,
                                 orient=VERTICAL,
                                 command=self.text_area.yview)
        self.text_area['yscrollcommand'] = self.scrollY.set
        self.scrollY.pack(side=RIGHT, expand=YES, fill=Y, anchor=W)

        # init stuff
        self.myParent.title("Total Open Station")
        self.myParent.protocol("WM_DELETE_WINDOW", self.on_app_close)
        self.print_model()
        self.myParent.mainloop()

    def on_click_language(self):
        '''
            open select language dialog
        '''
        pass

    def on_app_close(self):
        '''Callback function to ask for confirmation before quitting the application.'''

        if askokcancel("Quit","Do you really want to quit application ?"):
            self.myParent.destroy()

    def exit_action(self, event):
        self.on_app_close()

    def print_model(self):
        model = self.optionMODEL_value.get()
        if model != 'custom':
            self.control_panel.forget()
            self.option2_value.set(0)
            self.option3_value.set(0)
            self.option4_value.set('')
            self.option5_value.set(0)
        else:
            self.option2_value.set(9600)
            self.option3_value.set(8)
            self.option4_value.set('N')
            self.option5_value.set(1)
            self.control_panel.pack(side=TOP,
                                    expand=YES,
                                    fill=Y,
                                    ipadx=5,
                                    ipady=5)

    def connect(self):

        chosen_model = self.optionMODEL_value.get()
        chosen_port = self.option1_value.get()

        # do nothing if input is empty
        if not (chosen_model == '' or chosen_port == ''):

            if chosen_model == 'custom':

                # dictionary for passing options to Serial
                self.options = {'baudrate': self.option2_value.get(),
                                'bytesize': self.option3_value.get(),
                                'parity': self.option4_value.get(),
                                'stopbits': self.option5_value.get()}
            else:
                self.options = {}

            modelclass = BUILTIN_MODELS[chosen_model]
            if isinstance(modelclass, tuple):
                try:
                    # builtin model builder
                    mod, cls, name = modelclass
                    modelclass = getattr(
                        __import__('totalopenstation.models.' + mod, None, None, [cls]), cls)
                except ImportError as msg:
                    showwarning(_('Import error'),
                                _('Error loading the required model module: %s' % msg))

                mc = modelclass(chosen_port, **self.options)

                try:
                    mc.close()  # sometimes the port will be already open for no reason
                    mc.open()
                except serial.SerialException as detail:
                    e = ErrorDialog(self.myParent, detail)
                else:
                    st = DownloadDialog(self.myParent)
                    sleeptime = float(self.option6_value.get())
                    if st.result:
                        self.status.set(_("Waiting for data: Please start the transfer from your total station menu."))
                        while mc.inWaiting() == 0:
                            sleep(sleeptime)
                        n = mc.inWaiting()
                        result = mc.read(n)
                        self.replace_text(result.encode())
                        sleep(sleeptime)

                        while mc.inWaiting() > 0:
                            newdata = mc.read(mc.inWaiting())
                            result += newdata
                            self.status.set(_('Downloaded %d bytes'), len(result))
                            self.replace_text(result.encode())
                            sleep(sleeptime) # TODO determine sleep time from baudrate
                        mc.close()
                        showinfo(_('Success!'),
                                 _('Download finished!\nYou have %d bytes of data.') % len(result))

    def connect_action(self, event):
        self.connect()

    def open_a_file(self):
        try:
            d = tkinter.filedialog.askopenfilename()
            of = open(d, 'r')
            oc = of.read()
            self.replace_text(oc)
        except:
            pass

    def open_action(self, event):
        self.open_a_file()

    def process(self):
        data = self.text_area.get("1.0", END)
        d = ProcessDialog(self.myParent, data)

    def process_action(self, event):
        self.process()

    def save_a_file(self):
        try:
            sd = tkinter.filedialog.asksaveasfilename(defaultextension='.tops')
            data = self.text_area.get("1.0", END)
            of = open(sd, 'w')
            oc = of.write(data)
        except:
            pass

    def save_action(self, event):
        self.save_a_file()

    def about(self):
        d = AboutDialog(self.myParent)

    def about_action(self, event):
        self.about()

    def replace_text(self, text):
        self.text_area.delete("1.0", END)
        self.text_area.insert(END, text.replace('\r', ''))
        self.text_area.yview_moveto(1.0)
        self.text_area.update_idletasks()


root = Tk()
Tops = Tops(root)


#save user's preferences (model, port and sleeptime if custom model)

atexit.register(Tops.upref.setvalues,
                {'model': Tops.optionMODEL_value.get(),
                 'port': Tops.option1_value.get(),
                 'sleeptime': Tops.option6_value.get(),
                 })



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: upref.py
# Copyright 2019 Stefano Costa <steko@iosa.it>
# Copyright 2010 Luca Bianconi <luxetluc@yahoo.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import atexit
import logging
import os
import os.path

from configparser import ConfigParser, NoSectionError, NoOptionError

logger = logging.getLogger(__name__)

class UserPrefs(ConfigParser):
    '''Manage user preferences for GUI options and last used values.

    Proof-of-concept. Notes:

    * user preferences path is hardcoded here
    * returns a dictionary of user preferences
    * preferences will be set through a dictionary as well'''

    OPTIONS = {
        'model': '',
        'port': '',
        'sleeptime': '1.0',       # added in 0.3.1
    }

    def __init__(self):

        ConfigParser.__init__(self)

        USER_PREFS_PATH = '~/.totalopenstation/totalopenstation.cfg'

        self.upref = os.path.expanduser(USER_PREFS_PATH)

        try:
            with open(self.upref) as f:
                self.read(f)
        except FileNotFoundError:
                self.initfile()
        else:
            try:
                self.getvalue('model')
            except NoSectionError:
                self.initfile()


    def initfile(self):
        try:
            self.write()
        except FileNotFoundError:
            os.mkdir(os.path.dirname(self.upref))
        finally:
            self.write()
        logger.info('User preferences do not exist!')
        self.add_section('topsconfig')
        for k,v in list(self.OPTIONS.items()):
            self.set('DEFAULT', k, v)
        logger.info('Created new user preferences file with default values')

    def write(self):
        ''' override ConfigParser.write() method '''

        ConfigParser.write(self, open(self.upref, 'w'))

    def getdict(self):
        ''' get config file values '''

        current_options = {}
        for k in list(self.OPTIONS.keys()):
            current_options[k] = self.getvalue(k)

        return current_options

    def getvalue(self, option):
        ''' get specific config file value '''

        value = self.get('topsconfig', option)
        return value

    def setvalues(self, values):
        ''' set specific config file value '''

        for k, v in list(values.items()):
            self.set('topsconfig', str(k), str(v))

        self.write()



import sys

if not('.' in sys.path): sys.path.append('.')
import math

from Tkinter import *
from tkFileDialog import *
from tkSimpleDialog import askstring
import math

# thinking in tkinter http://www.ferg.org/thinking_in_tkinter/all_programs.html

class TheGui:
    def __init__(self, parent):
       
        #------- frmIn ----------#
        # http://effbot.org/tkinterbook/tkinter-widget-styling.htm
        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()

        self.lblS = Label(self.frmOut, text='Occupation point[grid coords]', bg = 'red', width=450)
        self.lblS.pack() 

        self.xs = Label(self.frmOut, text='x:', width=2)
        self.xs.pack(side=LEFT)
        self.xs = StringVar() # coord x of occupation point
        self.entSx = Entry(self.frmOut, width=20, textvariable=self.xs)
        self.xs.set('6020279.50')
        self.entSx.pack(side=LEFT)

        self.ys = Label(self.frmOut, text='y:', width=2)
        self.ys.pack(side=LEFT)
        self.ys = StringVar() # coord y of occupation point
        self.entSy = Entry(self.frmOut, width=20, textvariable=self.ys)
        self.entSy.pack(side=LEFT)
        self.ys.set('6430432.70')

        

   ###########################################################################################     
        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()

        self.lblOut = Label(self.frmOut, text='Base point[grid coords]', bg = 'red', width=450)
        self.lblOut.pack() 

        self.xa = Label(self.frmOut, text='x:', width=2)
        self.xa.pack(side=LEFT)
        self.xa = StringVar() # coord x of base point
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.xa)
        self.entIn.pack(side=LEFT)
        self.xa.set('6020273.90')

        self.ya = Label(self.frmOut, text='y:', width=2)
        self.ya.pack(side=LEFT)
        self.ya = StringVar() # coord y of base point
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.ya)
        self.entIn.pack(side=LEFT)
        self.ya.set('6430341.40')



##################################################################################################
        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()

        self.lblOut = Label(self.frmOut, text='Control point[grid coords]', bg = 'red', width=450)
        self.lblOut.pack() 

        self.xb = Label(self.frmOut, text='x:', width=2)
        self.xb.pack(side=LEFT)
        self.xb = StringVar() # coord x of control point
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.xb)
        self.entIn.pack(side=LEFT)
        self.xb.set('6020296.10')

        self.yb = Label(self.frmOut, text='y:', width=2)
        self.yb.pack(side=LEFT)
        self.yb = StringVar() # coord y of control point
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.yb)
        self.entIn.pack(side=LEFT)
        self.yb.set('6430482.80')
        
        
############################################################################
        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()
        
        
        self.AngleSA = Label(self.frmOut, text='Base point angle[gons]:', width=20)
        self.AngleSA.pack(side=LEFT)
        self.AngleSA = StringVar()
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.AngleSA)
        self.entIn.pack(side=LEFT)
        self.AngleSA.set('0.0000')


        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()
        
        
        
        self.AngleSB = Label(self.frmOut, text='Control point angle[gons]:', width=20)
        self.AngleSB.pack(side=LEFT)
        self.AngleSB = StringVar()
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.AngleSB)
        self.entIn.pack(side=LEFT)
        self.AngleSB.set('183.5311')


        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()
        
        
        
        self.AngleSP = Label(self.frmOut, text='Calculated point angle[gons]', width=20)
        self.AngleSP.pack(side=LEFT)
        self.AngleSP = StringVar()
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.AngleSP)
        self.entIn.pack(side=LEFT)
        self.AngleSP.set('91.2643')


        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()             
        
        self.dSP = Label(self.frmOut, text='Horizontal distance[m]:', width=20)
        self.dSP.pack(side=LEFT)
        self.dSP = StringVar()
        self.entIn = Entry(self.frmOut, width=20, text='x', textvariable=self.dSP)
        self.entIn.pack(side=LEFT)
        self.dSP.set('54.26')
        


        
        
        #------- frmButtons ----------#
        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()
        
        self.btnConvert = Button(self.frmOut, 
            text='Calculate', command=self.btnConvertClick)
        self.btnConvert.pack()
        

        self.frmOut = Frame(parent, bd=5)
        self.frmOut.pack()

        self.lblOut = Label(self.frmOut, text='Calculated point coordinates',bg = 'red', width=450)
        self.lblOut.pack() 

        self.xp = Label(self.frmOut, text='x:', width=2)
        self.xp.pack(side=LEFT)
        self.xp = StringVar()
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.xp)
        self.entIn.pack(side=LEFT)

        self.yp = Label(self.frmOut, text='y:', width=2)
        self.yp.pack(side=LEFT)
        self.yp = StringVar()
        self.entIn = Entry(self.frmOut, width=20, textvariable=self.yp)
        self.entIn.pack(side=LEFT)
  
    #------- handle commands ----------#
            
    def btnConvertClick(self):
        xs = float(self.xs.get())
        ys = float(self.ys.get())

        xa = float(self.xa.get())
        ya = float(self.ya.get())

        xb = float(self.xb.get())
        yb = float(self.yb.get())


        ka = float(self.AngleSA.get())
        kb = float(self.AngleSB.get())
        kp = float(self.AngleSP.get())

        dsp = float(self.dSP.get())

        pi = math.pi

        dx = xa - xs
        dy = ya - ys

        bdx = xb - xs
        bdy = yb - ys

        n = dy/dx# in radians

        if dy ==0:
            if dy>0:
                A1 = pi/2
            else:
                A1 = 1.5 * pi
        else:
            A1 = math.atan(dy/dx)
            if dx < 0:
                A1 = A1 + pi
            elif dy < 0:
                A1 = A1 + 2 * pi

        dd = A1 *200/pi

        if bdy ==0:
            if bdy>0:
                A2 = pi/2
            else:
                A2 = 1.5 * pi
        else:
            A2 = math.atan(bdy/bdx)
            if bdx < 0:
                A2 = A2 + pi
            elif bdy < 0:
                A2 = A2 + 2 * pi
        ss = A2 *200/pi

        y1 = (A1 * 200/pi) - ka
        y2 = (A2 * 200/pi) - kb
        if y2 < 0:
            y2 = y2 +400
    

        Y = (y1 + y2)/2

        A3 = (Y + kp)
        aa = A3 * pi/200.0

        xp = xs + dsp * math.cos(aa)
        yp = ys + dsp * math.sin(aa)

        print xp, yp

        self.xp.set(str(xp))
        self.yp.set(str(yp))
        
        
 
  
root = Tk()
root.title("Obliczenia biegunowe v.0.2")
#http://infohost.nmt.edu/tcc/help/pubs/tkinter/std-attrs.html#geometry
#http://infohost.nmt.edu/tcc/help/pubs/tkinter/toplevel.html
root.geometry("450x500+10+10")
gui = TheGui(root)
root.mainloop()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: tops_txt.py
# Copyright 2008 Luca Bianconi<lc.bianconi@googlemail.com>
# Copyright 2019 Stefano Costa <steko@iosa.it>
#
# This file is part of Total Open Station.
#
# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

from . import Builder


def to_txt(d):
    try:
        string = "{geom.x} {geom.y} {geom.z}\n".format(geom=d.geometry)
    except ValueError:
        string = "{geom.x} {geom.y}\n".format(geom=d.geometry)
    return string

class OutputFormat(Builder):

    """
    Exports points data in TXT (space-separated) format line by line.

    ``data`` should be an iterable of Feature objects.
    """

    def __init__(self, data):
        self.data = data

    def process(self):
        lines = [to_txt(e) for e in self.data]
        output = "".join(lines)
        return output



import unittest

from totalopenstation.formats import Point
from totalopenstation.formats.polar import BasePoint, PolarPoint


class TestPolar(unittest.TestCase):

    def setUp(self):
        self.bp0 = BasePoint(x='0', y='0', z='0', ih='1.0', b_zero_st='0.0')
        self.bp1 = BasePoint(x='0', y='0', z='0', ih='1.324', b_zero_st='0.0')

        self.p0 = PolarPoint(angle_unit='deg',
                             z_angle_type='z',
                             dist_type='s',
                             dist=9,
                             angle=180,
                             z_angle=90,
                             th=0,
                             base_point=self.bp0,
                             pid=1,
                             text='Test Point',
                             coordorder='ENZ')

        self.p1 = PolarPoint(angle_unit='gon',
                             z_angle_type='z',
                             dist_type='s',
                             dist=24.567,
                             angle=34.120,
                             z_angle=100,
                             th=1.500,
                             base_point=self.bp0,
                             pid=2,
                             text='Real Point',
                             coordorder='NEZ')

        self.p2 = PolarPoint(angle_unit='dms',
                             z_angle_type='z',
                             dist_type='s',
                             dist=13.825,
                             angle=35.45100,
                             z_angle=91.17510,
                             th=1.300,
                             base_point=self.bp1,
                             pid=3,
                             text='Real Point',
                             coordorder='ENZ')

    def test_polar(self):
        p0_test = Point(0.0, -9.0, 1.0)
        self.assertAlmostEqual(self.p0.to_point().x, p0_test.x)
        self.assertAlmostEqual(self.p0.to_point().y, p0_test.y)
        self.assertAlmostEqual(self.p0.to_point().z, p0_test.z)

        p1_test = Point(21.1222392859, 12.5454572076, -0.5)
        self.assertAlmostEqual(self.p1.to_point().x, p1_test.x)
        self.assertAlmostEqual(self.p1.to_point().y, p1_test.y)
        self.assertAlmostEqual(self.p1.to_point().z, p1_test.z)

        p2_test = Point(8.0757244, 11.21674196, -0.2890493)
        self.assertAlmostEqual(self.p2.to_point().x, p2_test.x)
        self.assertAlmostEqual(self.p2.to_point().y, p2_test.y)
        self.assertAlmostEqual(self.p2.to_point().z, p2_test.z)



# -*- coding: utf-8 -*-
# filename: tops_geojson.py
# Copyright 2009 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import json

from totalopenstation.formats import FeatureCollection

from . import Builder


class OutputFormat(Builder):
    '''A GeoJSON output driver.'''

    def __init__(self, data):

        self.feature_collection = FeatureCollection(data)

    def process(self):

        return json.dumps(self.feature_collection.__geo_interface__)



import ctypes
import re

def ValidHandle(value):
    if value == 0:
        raise ctypes.WinError()
    return value

NULL = 0
HDEVINFO = ctypes.c_int
BOOL = ctypes.c_int
CHAR = ctypes.c_char
PCTSTR = ctypes.c_char_p
HWND = ctypes.c_uint
DWORD = ctypes.c_ulong
PDWORD = ctypes.POINTER(DWORD)
ULONG = ctypes.c_ulong
ULONG_PTR = ctypes.POINTER(ULONG)
#~ PBYTE = ctypes.c_char_p
PBYTE = ctypes.c_void_p

class GUID(ctypes.Structure):
    _fields_ = [
        ('Data1', ctypes.c_ulong),
        ('Data2', ctypes.c_ushort),
        ('Data3', ctypes.c_ushort),
        ('Data4', ctypes.c_ubyte*8),
    ]
    def __str__(self):
        return "{%08x-%04x-%04x-%s-%s}" % (
            self.Data1,
            self.Data2,
            self.Data3,
            ''.join(["%02x" % d for d in self.Data4[:2]]),
            ''.join(["%02x" % d for d in self.Data4[2:]]),
        )

class SP_DEVINFO_DATA(ctypes.Structure):
    _fields_ = [
        ('cbSize', DWORD),
        ('ClassGuid', GUID),
        ('DevInst', DWORD),
        ('Reserved', ULONG_PTR),
    ]
    def __str__(self):
        return "ClassGuid:%s DevInst:%s" % (self.ClassGuid, self.DevInst)
PSP_DEVINFO_DATA = ctypes.POINTER(SP_DEVINFO_DATA)

class SP_DEVICE_INTERFACE_DATA(ctypes.Structure):
    _fields_ = [
        ('cbSize', DWORD),
        ('InterfaceClassGuid', GUID),
        ('Flags', DWORD),
        ('Reserved', ULONG_PTR),
    ]
    def __str__(self):
        return "InterfaceClassGuid:%s Flags:%s" % (self.InterfaceClassGuid, self.Flags)
PSP_DEVICE_INTERFACE_DATA = ctypes.POINTER(SP_DEVICE_INTERFACE_DATA)

PSP_DEVICE_INTERFACE_DETAIL_DATA = ctypes.c_void_p

SetupDiDestroyDeviceInfoList = ctypes.windll.setupapi.SetupDiDestroyDeviceInfoList
SetupDiDestroyDeviceInfoList.argtypes = [HDEVINFO]
SetupDiDestroyDeviceInfoList.restype = BOOL

SetupDiGetClassDevs = ctypes.windll.setupapi.SetupDiGetClassDevsA
SetupDiGetClassDevs.argtypes = [ctypes.POINTER(GUID), PCTSTR, HWND, DWORD]
SetupDiGetClassDevs.restype = ValidHandle #HDEVINFO

SetupDiEnumDeviceInterfaces = ctypes.windll.setupapi.SetupDiEnumDeviceInterfaces
SetupDiEnumDeviceInterfaces.argtypes = [HDEVINFO, PSP_DEVINFO_DATA, ctypes.POINTER(GUID), DWORD, PSP_DEVICE_INTERFACE_DATA]
SetupDiEnumDeviceInterfaces.restype = BOOL

SetupDiGetDeviceInterfaceDetail = ctypes.windll.setupapi.SetupDiGetDeviceInterfaceDetailA
SetupDiGetDeviceInterfaceDetail.argtypes = [HDEVINFO, PSP_DEVICE_INTERFACE_DATA, PSP_DEVICE_INTERFACE_DETAIL_DATA, DWORD, PDWORD, PSP_DEVINFO_DATA]
SetupDiGetDeviceInterfaceDetail.restype = BOOL

SetupDiGetDeviceRegistryProperty = ctypes.windll.setupapi.SetupDiGetDeviceRegistryPropertyA
SetupDiGetDeviceRegistryProperty.argtypes = [HDEVINFO, PSP_DEVINFO_DATA, DWORD, PDWORD, PBYTE, DWORD, PDWORD]
SetupDiGetDeviceRegistryProperty.restype = BOOL


GUID_CLASS_COMPORT = GUID(0x86e0d1e0, 0x8089, 0x11d0,
    (ctypes.c_ubyte*8)(0x9c, 0xe4, 0x08, 0x00, 0x3e, 0x30, 0x1f, 0x73))

DIGCF_PRESENT = 2
DIGCF_DEVICEINTERFACE = 16
INVALID_HANDLE_VALUE = 0
ERROR_INSUFFICIENT_BUFFER = 122
SPDRP_HARDWAREID = 1
SPDRP_FRIENDLYNAME = 12
ERROR_NO_MORE_ITEMS = 259

def comports(available_only=True):
    """This generator scans the device registry for com ports and yields port, desc, hwid.
       If available_only is true only return currently existing ports."""
    flags = DIGCF_DEVICEINTERFACE
    if available_only:
        flags |= DIGCF_PRESENT
    g_hdi = SetupDiGetClassDevs(ctypes.byref(GUID_CLASS_COMPORT), None, NULL, flags);
    #~ for i in range(256):
    for dwIndex in range(256):
        did = SP_DEVICE_INTERFACE_DATA()
        did.cbSize = ctypes.sizeof(did)

        if not SetupDiEnumDeviceInterfaces(
            g_hdi,
            None,
            ctypes.byref(GUID_CLASS_COMPORT),
            dwIndex,
            ctypes.byref(did)
        ):
            if ctypes.GetLastError() != ERROR_NO_MORE_ITEMS:
                raise ctypes.WinError()
            break
        
        dwNeeded = DWORD()
        # get the size
        if not SetupDiGetDeviceInterfaceDetail(
            g_hdi,
            ctypes.byref(did),
            None, 0, ctypes.byref(dwNeeded),
            None
        ):
            # Ignore ERROR_INSUFFICIENT_BUFFER
            if ctypes.GetLastError() != ERROR_INSUFFICIENT_BUFFER:
                raise ctypes.WinError()
        # allocate buffer
        class SP_DEVICE_INTERFACE_DETAIL_DATA_A(ctypes.Structure):
            _fields_ = [
                ('cbSize', DWORD),
                ('DevicePath', CHAR*(dwNeeded.value - ctypes.sizeof(DWORD))),
            ]
            def __str__(self):
                return "DevicePath:%s" % (self.DevicePath,)
        idd = SP_DEVICE_INTERFACE_DETAIL_DATA_A()
        idd.cbSize = 5
        devinfo = SP_DEVINFO_DATA()
        devinfo.cbSize = ctypes.sizeof(devinfo)
        if not SetupDiGetDeviceInterfaceDetail(
            g_hdi,
            ctypes.byref(did),
            ctypes.byref(idd), dwNeeded, None,
            ctypes.byref(devinfo)
        ):
            raise ctypes.WinError()

        # hardware ID
        szHardwareID = ctypes.create_string_buffer('\0' * 250)
        if not SetupDiGetDeviceRegistryProperty(
            g_hdi,
            ctypes.byref(devinfo),
            SPDRP_HARDWAREID,
            None,
            ctypes.byref(szHardwareID), ctypes.sizeof(szHardwareID) - 1,
            None
        ):
            # Ignore ERROR_INSUFFICIENT_BUFFER
            if ctypes.GetLastError() != ERROR_INSUFFICIENT_BUFFER:
                raise ctypes.WinError()

        # friendly name
        szFriendlyName = ctypes.create_string_buffer('\0' * 250)
        if not SetupDiGetDeviceRegistryProperty(
            g_hdi,
            ctypes.byref(devinfo),
            SPDRP_FRIENDLYNAME,
            None,
            ctypes.byref(szFriendlyName), ctypes.sizeof(szFriendlyName) - 1,
            None
        ):
            # Ignore ERROR_INSUFFICIENT_BUFFER
            if ctypes.GetLastError() != ERROR_INSUFFICIENT_BUFFER:
                raise ctypes.WinError()
        port_name = re.search(r"\((.*)\)", szFriendlyName.value).group(1)
        if len(port_name) > 4:
            port_name = '\\\\.\\'+port_name
        yield port_name, szFriendlyName.value, szHardwareID.value
    
    SetupDiDestroyDeviceInfoList(g_hdi)


if __name__ == '__main__':
    import serial
    for port, desc, hwid in comports():
        print("%s: %s (%s)" % (port, desc, hwid))
        print(" "*10, serial.Serial(port)) #test open
    
    # list of all ports the system knows
    print("-"*60)
    for port, desc, hwid in comports(False):
        print("%-10s: %s (%s)" % (port, desc, hwid))



# -*- coding: utf-8 -*-
# filename: formats/leica_gsi.py
# Copyright 2012 Stefano Costa <steko@iosa.it>
# Copyright 2015-2016 Damien Gaignon <damien.gaignon@gmail.com>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.

import logging
 
from . import Feature, Parser, Point, UNKNOWN_STATION, UNKNOWN_POINT
from .polar import BasePoint, PolarPoint

# Distance units depend of the last digit
# 0, 6 and 8 are in mm, 1/10mm and 1/100mm
# 1 and 7 are in ft and 1/10000ft converted in m
UNITS = {"angle": {'21', '22', '25'},
         "distance": {'31', '32', '33', '81', '84', '87', '88'},
         "2": "gon", "3": "deg", "4": "dms", "5": "mil",
         "0": "meter", "1": "feet", "6": "dmeter", "7": "dfeet", "8": "mmeter",
         "gon": 100000, "deg": 100000, "dms": 100000, "mil": 10000,
         "meter": 1000, "feet": 1000, "dmeter": 10000, "dfeet": 10000, "mmeter": 100000}

logger = logging.getLogger(__name__)

class FormatParser(Parser):
    '''The FormatParser for Leica GSI data format.

    Args:
        data (str): A string representing the file to be parsed.
            
    Attributes:
        line (list): A list of each lines of the file being parsed.
    '''

    def __init__(self, data):
        self.rows = data.splitlines()

    def _get_comments(self):
        """
        Get all comments of the parsed line
        
        Returns:
            A list of comments or an empty list
        """
        comments = []
        for i in range(1, 10):
            try:
                comments.append(self.tdict['4%s' % i]['data'].lstrip('0'))
            except KeyError:
                break

        return comments

    def _get_attrib(self):
        """
        Get all attributes or remarks of the parsed line
        
        Returns:
            A list of attributes and remarks or an empty list
        """
        attrib = []
        for i in range(1, 10):
            try:
                attrib.append(self.tdict['7%s' % i]['data'].lstrip('0'))
            except KeyError:
                break

        return attrib

    def _get_coordinates(self, first_coor, unit):
        """
        Get all coordinates of the parsed line
        
        Args:
            first_coor (str): The Word Index of the first coordinate.
                Could be 81 or 84
            unit (int): The divider corresponding to the coordinate unit.

        Returns:
            All three coordinates in the forme of X, Y, Z
            If these coordinates do not exist, return None
        """
        second_coor = str(int(first_coor) + 1)
        third_coor = str(int(first_coor) + 2)
        try:
            x_sign, x_data = self.tdict[first_coor]['sign'], self.tdict[first_coor]['data']
            y_sign, y_data = self.tdict[second_coor]['sign'], self.tdict[second_coor]['data']
            z_sign, z_data = self.tdict[third_coor]['sign'], self.tdict[third_coor]['data']
        except KeyError:
            x = None
            y = None
            z = None
        else:
            x = float(x_sign + x_data) / unit
            y = float(y_sign + y_data) / unit
            z = float(z_sign + z_data) / unit

        return x, y, z

    def _get_angle(self, angle, unit):
        """
        Get an angle of the parsed line
        
        Returns:
            A floating number representing the angle
            If this angle does not exist, return None
            
        """
        try:
            angle_sign, angle_data = self.tdict[angle]['sign'], self.tdict[angle]['data']
        except KeyError:
            angle = None
        else:
            angle = float(angle_sign + angle_data) / unit

        return angle

    def _get_edm_accuracy(self, ldata):
        """
        Get the ppm and the prism constant of the parsed line
        
        Returns:
            Two floating numbers representing ppm and prism constant
            If these values do not exist, return None
        """
        try:
            ppm_sign, ppm_data = self.tdict['51']['sign'], self.tdict['51']['data'][:ldata-4]
            pc_sign, pc_data = self.tdict['51']['data'][ldata-4], self.tdict['51']['data'][ldata-3:]
        except KeyError:
            try:
                ppm_sign, ppm_data = self.tdict['59']['sign'], self.tdict['59']['data']
                pc_sign, pc_data = self.tdict['58']['sign'], self.tdict['58']['data']
            except KeyError:
                ppm = None
                prism_constant = None
            else:
                ppm = float(ppm_sign + ppm_data)
                prism_constant = float(pc_sign + pc_data)
        else:
            ppm = float(ppm_sign + ppm_data)
            prism_constant = float(pc_sign + pc_data)

        return ppm, prism_constant

    def _get_value(self, value, unit):
        """
        Get a value of the parsed line
        
        Returns:
            A string value or None
        """
        try:
            value_sign, value_data = self.tdict[value]['sign'], self.tdict[value]['data']
        except KeyError:
            value = None
        else:
            value = float(value_sign + value_data) / unit

        return value

    @property
    def points(self):
        '''Extract all GSI data.

        This parser is based on the information in :ref:`if_leica_gsi`
        
        Returns:
            A list of GeoJSON-like Feature object representing points coordinates.
    
        Raises:
            KeyError: An error occured during line read, this line could not be 
                computed as the WI does not exist.
            KeyError: An error occured during computation, the data does not exist.
        
        Notes:
            Information needed are:
                - station : 11, 84, 85, 86, 88
                - direct point : 11, 81, 82, 83
                - computed point : 11, 21, 22, 31 or 32, 87 [, 88] [, 81, 82, 83]
            Angles are considered as zenithal
        '''
        
        points = []
        bp = None
        for row in self.rows:
            tokens = row.split()
            self.tdict = {}
            for t in tokens:
                t = t.lstrip('*')
                data = {
                    'wordindex': t[0:2],
                    'info': t[2:6],
                    'sign': t[6],
                    'data': t[7:],
                }
                self.tdict[data['wordindex']] = data

            try:
                pid = int(self.tdict['11']['info'])
                text = self.tdict['11']['data'].lstrip('0')
            except KeyError:
                pass
            else:
                # Get angle and distance units
                try:
                    angle_code = list(UNITS['angle'] & set(self.tdict.keys()))[0]
                    angle_unit = UNITS[self.tdict[angle_code]['info'][3]]
                except IndexError:
                    pass
                try:
                    dist_code = list(UNITS['distance'] & set(self.tdict.keys()))[0]
                    dist_unit = UNITS[self.tdict[dist_code]['info'][3]]
                except IndexError:
                    pass
                # Beginning of the parsing
                try:
                    # Look for point coordinates
                    x, y, z = self.tdict['81'], self.tdict['82'], self.tdict['83']
                except KeyError:
                    try:
                        angle, z_angle = self.tdict['21'], self.tdict['22']
                        z_angle_type = 'z'
                        # 31 or/and 32
                        try:
                            dist = self.tdict['31']
                        except KeyError:
                            try:
                                dist = self.tdict['32']
                            except KeyError:
                                logger.info('There is no distance value')
                                dist = None
                            else:
                                dist_type = 'h'
                        else:
                            dist_type = 's'
                        th = self.tdict['87']
                    except KeyError:
                        try:
                            # Look for a station
                            x, y, z = self.tdict['84'], self.tdict['85'], self.tdict['86']
                            ih = self.tdict['88']
                        except KeyError:
                            pass
                        else:
                            # Compute station data
                            x, y, z = self._get_coordinates("84", UNITS[dist_unit])
                            ih = self._get_value("88", UNITS[dist_unit])
                            bp = BasePoint(x=x, y=y, z=z, ih=ih, b_zero_st=0.0)
                            p = Point(x, y, z)
                            f = Feature(p,
                                        desc='ST',
                                        id=pid,
                                        point_name=text,
                                        dist_unit=dist_unit)
                            points.append(f)
                    else:
                        angle = self._get_angle("21", UNITS[angle_unit])
                        z_angle = self._get_angle("22", UNITS[angle_unit])
                        if dist_type == 's':
                            dist = self._get_value("31", UNITS[dist_unit])
                        else:
                            dist = self._get_value("32", UNITS[dist_unit])
                        th = self._get_value("87", UNITS[dist_unit])
                        # Polar data may have point coordinates (not used)
                        x, y, z = self._get_coordinates("81", UNITS[dist_unit])
                        # Polar data may have instrument height
                        ih = self._get_value("88", UNITS[dist_unit])
                        if ih is None:
                            ih = 0.0
                        if bp is None:
                            bp = BasePoint(x=0.0, y=0.0, z=0.0, ih=ih, b_zero_st=0.0)
                        p = PolarPoint(angle_unit=angle_unit,
                                       z_angle_type=z_angle_type,
                                       dist_type=dist_type,
                                       dist=dist,
                                       angle=angle,
                                       z_angle=z_angle,
                                       th=th,
                                       base_point=bp,
                                       pid=pid,
                                       text=text,
                                       coordorder='ENZ')
                        f = Feature(p.to_point(),
                                    desc='PT',
                                    id=pid,
                                    point_name=text)
                        points.append(f)
                else:
                    x, y, z = self._get_coordinates("81", UNITS[dist_unit])
                    p = Point(x, y, z)
                    f = Feature(p,
                                desc='PT',
                                id=pid,
                                point_name=text)
                    points.append(f)
        logger.debug(points)
        return points

    @property
    def raw_line(self):
        '''Extract all GSI data.

        This parser is based on the information in :ref:`if_leica_gsi`
        
        Returns:
            A list of GeoJSON-like Feature object representing raw data
                i.e. polar coordinates and other informations.
    
        Raises:
            KeyError: An error occured during line read, this line could not be 
                computed as the WI does not exist.
            KeyError: An error occured during computation, the data does not exist.
        
        Notes:
            Information needed are:
                - station : 11 [, 25], 84, 85, 86 [, 87], 88
                - direct point : 11, 81, 82, 83
                - computed point : 11, 21, 22, 31 or 32 [, 51], 87 [, 88] [, 81, 82, 83]
            Angles are considered as zenithal
        '''

        points = []
        # GSI files handles 8 or 16 bits data block. This will check the size
        ldata = len(self.rows[0].split()[0].lstrip('*')[7:])
        station_id = 1

        for row in self.rows:
            tokens = row.split()
            self.tdict = {}
            for t in tokens:
                t = t.lstrip('*')
                data = {
                    'wordindex': t[0:2],
                    'info': t[2:6],
                    'sign': t[6],
                    'data': t[7:],
                }
                self.tdict[data['wordindex']] = data

            try:
                pid = int(self.tdict['11']['info'])
                point_name = self.tdict['11']['data'].lstrip('0')
            except KeyError:
                try:
                    comments = self.tdict['41']
                except KeyError:
                    logger.info("The line %s will not be computed as the code '%s' is not known"\
                          % (pid, row[0:2]))
                else:
                    # Compute comments
                    comments = self._get_comments()
            else:
                # Get angle and distance units
                try:
                    angle_code = list(UNITS['angle'] & set(self.tdict.keys()))[0]
                    angle_unit = UNITS[self.tdict[angle_code]['info'][3]]
                except IndexError:
                    pass
                try:
                    dist_code = list(UNITS['distance'] & set(self.tdict.keys()))[0]
                    dist_unit = UNITS[self.tdict[dist_code]['info'][3]]
                except IndexError:
                    pass
                # Beginning of the parsing
                try:
                    # Look for a station
                    x, y, z = self.tdict['84'], self.tdict['85'], self.tdict['86']
                    ih = self.tdict['88']
                except KeyError:
                    # Otherwise look for polar data
                    try:
                        angle, z_angle = self.tdict['21'], self.tdict['22']
                        # 31 or/and 32
                        try:
                            dist = self.tdict['31']
                        except KeyError:
                            try:
                                dist = self.tdict['32']
                            except KeyError:
                                logger.info('There is no distance value')
                                dist = None
                            else:
                                dist_type = 'h'
                        else:
                            dist_type = 's'
                        th = self.tdict['87']
                    except KeyError:
                        # Otherwise look for point coordinates only
                        try:
                            x, y, z = self.tdict['81'], self.tdict['82'], self.tdict['83']
                        except KeyError:
                            # Otherwise look for Remark or Attrib
                            try:
                                attrib = self.tdict['71']
                            except KeyError:
                                # No more possibilities
                                logger.info("These data can not be compute : %s" % (self.tdict))
                            else:
                                # Compute remark or Attrib
                                attrib = self._get_attrib()
                        else:
                            # Compute point coordinates
                            x, y, z = self._get_coordinates("81", UNITS[dist_unit])
                            # Point coordinates may have remarks or attributes
                            attrib = self._get_attrib()

                            if x:
                                p = Point(x, y, z)
                            else:
                                logger.info('There is no known point')
                                p = UNKNOWN_POINT
                            f = Feature(p,
                                        desc='PT',
                                        id=pid,
                                        point_name=point_name,
                                        dist_unit=dist_unit,
                                        attrib=attrib)
                            points.append(f)
                    else:
                        # Compute polar data
                        angle = self._get_angle("21", UNITS[angle_unit])
                        z_angle = self._get_angle("22", UNITS[angle_unit])
                        z_angle_type = 'z'
                        if dist_type == 's':
                            dist = self._get_value("31", UNITS[dist_unit])
                        else:
                            dist = self._get_value("32", UNITS[dist_unit])
                        th = self._get_value("87", UNITS[dist_unit])
                        # Polar data may have point coordinates
                        x, y, z = self._get_coordinates("81",UNITS[dist_unit])
                        # Polar data may have instrument height
                        ih = self._get_value("88", UNITS[dist_unit])
                        # Polar data may have constant data
                        ppm, prism_constant = self._get_edm_accuracy(ldata)
                        # Polar data may have remarks or attributes
                        attrib = self._get_attrib()

                        if x:
                            p = Point(x, y, z)
                        else:
                            logger.info('There is no known point')
                            p = UNKNOWN_POINT

                        try:
                            station_name
                        except UnboundLocalError:
                            logger.info('There is no known station')
                            station_name = 'station_' + str(station_id)
                            station_id += 1
                        f = Feature(p,
                                    desc='PO',
                                    id=pid,
                                    point_name=point_name,
                                    angle_unit=angle_unit,
                                    z_angle_type=z_angle_type,
                                    dist_unit=dist_unit,
                                    dist_type=dist_type,
                                    angle=angle,
                                    z_angle=z_angle,
                                    dist=dist,
                                    th=th,
                                    ih=ih,
                                    ppm=ppm,
                                    prism_constant=prism_constant,
                                    st_name=station_name,
                                    attrib=attrib)
                        points.append(f)
                else:
                    # Compute station data
                    x, y, z = self._get_coordinates("84", UNITS[dist_unit])
                    ih = self._get_value("88", UNITS[dist_unit])
                    # Station data may have an azimuth angle
                    hz0 = self._get_angle("25", UNITS[angle_unit])
                    # Station data may have remarks or attributes
                    attrib = self._get_attrib()

                    if x:
                        p = Point(x, y, z)
                        station_name = point_name
                    else:
                        logger.info('There is no known station')
                        p = UNKNOWN_STATION
                        station_name = "station_" + str(station_id)
                        station_id += 1
                    f = Feature(p,
                                desc='ST',
                                id=pid,
                                point_name=point_name,
                                angle_unit=angle_unit,
                                dist_unit=dist_unit,
                                ih=ih,
                                hz0=hz0,
                                attrib=attrib)
                    points.append(f)

        logger.debug(points)
        return points



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# filename: totalopenstation-cli-connector.py
# Copyright 2008,2011 Stefano Costa <steko@iosa.it>

# This file is part of Total Open Station.

# Total Open Station is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# Total Open Station is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Total Open Station.  If not, see
# <http://www.gnu.org/licenses/>.


import gettext
import sys
import os

from optparse import OptionParser

import serial

from totalopenstation.models import BUILTIN_MODELS


t = gettext.translation('totalopenstation', './locale', fallback=True)
_ = t.gettext

usage = "usage: %prog [option] arg1 [option] arg2 ..."

parser = OptionParser(usage=usage)
parser.add_option("-m",
                "--model",
                action="store",
                type="string",
                dest="model",
                help="select input MODEL",
                metavar="MODEL")
parser.add_option("-p",
                "--port",
                action="store",
                type="string",
                dest="port",
                help="select input SERIAL PORT",
                metavar="PORT")
parser.add_option("-o",
                "--outfile",
                action="store",
                type="string",
                dest="outfile",
                help="select output FILE (do not specify for stdout)",
                metavar="FILE")

(options, args) = parser.parse_args()

if not (options.model and options.port):
    sys.exit("Please specify your model and the port to download from")

modelclass = BUILTIN_MODELS[options.model]

# import input format parser
if isinstance(modelclass, tuple):
    try:
        # builtin format parser
        mod, cls, name = modelclass
        modelclass = getattr(
            __import__('totalopenstation.models.' + mod, None, None, [cls]), cls)
    except ImportError as msg:
        sys.exit(_('Error loading the required model module: %s' % msg))

station = modelclass(options.port)
try:
    station.close()  # sometimes the port will be already open for no reason
    station.open()
except serial.SerialException as detail:
    sys.exit(detail)

print("Now you can start download from %s device" % options.model)

station.start()
station.dl_started.wait()
print("Download started...")
station.dl_finished.wait()
print("Download finished...")
result = station.result

if options.outfile:
    if not os.path.exists(options.outfile):
        e = open(options.outfile, 'w')
        e.write(result)
        e.close()
        print("Downloaded data saved to out file %s" % options.outfile)
    else:
        sys.exit("Specified output file already exists\n")
else:
    sys.stdout.write(result)
